@@ -35,7 +35,6 @@
 #include "provideclassinfo.h"
 #include "connectioncontainer.h"
 #include "objectsafety.h"
-#include "vlccontrol.h"
 #include "vlccontrol2.h"
 #include "viewobject.h"
 #include "dataobject.h"
@@ -50,7 +49,6 @@
 #include <servprov.h>
 #include <shlwapi.h>
 #include <wininet.h>
-#include <assert.h>
 
 using namespace std;
 
@@ -219,10 +217,9 @@ extern HMODULE DllGetModule();
 
 VLCPlugin::VLCPlugin(VLCPluginClass *p_class, LPUNKNOWN pUnkOuter) :
     _inplacewnd(NULL),
-    _WindowsManager(DllGetModule(), _ViewRC, this),
+    _WindowsManager(DllGetModule(), _ViewRC, &m_player, this),
     _p_class(p_class),
     _i_ref(1UL),
-    _p_libvlc(NULL),
     _i_codepage(CP_ACP),
     _b_usermode(TRUE)
 {
@@ -265,7 +262,6 @@ VLCPlugin::VLCPlugin(VLCPluginClass *p_class, LPUNKNOWN pUnkOuter) :
     vlcProvideClassInfo = new VLCProvideClassInfo(this);
     vlcConnectionPointContainer = new VLCConnectionPointContainer(this);
     vlcObjectSafety = new VLCObjectSafety(this);
-    vlcControl = new VLCControl(this);
     vlcControl2 = new VLCControl2(this);
     vlcViewObject = new VLCViewObject(this);
     vlcDataObject = new VLCDataObject(this);
@@ -280,24 +276,17 @@ VLCPlugin::VLCPlugin(VLCPluginClass *p_class, LPUNKNOWN pUnkOuter) :
 
     // make sure that persistable properties are initialized
     onInit();
-};
+
+    initVLC();
+}
 
 VLCPlugin::~VLCPlugin()
 {
-    if( vlc_player::is_open() )
-    {
-        if( isPlaying() )
-            playlist_stop();
-
-        player_unregister_events();
-    }
-
     delete vlcSupportErrorInfo;
     delete vlcOleObject;
     delete vlcDataObject;
     delete vlcViewObject;
     delete vlcControl2;
-    delete vlcControl;
     delete vlcConnectionPointContainer;
     delete vlcProvideClassInfo;
     delete vlcPersistPropertyBag;
@@ -314,10 +303,8 @@ VLCPlugin::~VLCPlugin()
     SysFreeString(_bstr_mrl);
     SysFreeString(_bstr_baseurl);
 
-    if( _p_libvlc )  { libvlc_release(_p_libvlc); _p_libvlc=NULL; }
-
     _p_class->Release();
-};
+}
 
 STDMETHODIMP VLCPlugin::QueryInterface(REFIID riid, void **ppv)
 {
@@ -350,15 +337,9 @@ STDMETHODIMP VLCPlugin::QueryInterface(REFIID riid, void **ppv)
         *ppv = reinterpret_cast<LPVOID>(vlcProvideClassInfo);
     else if( IID_IConnectionPointContainer == riid )
         *ppv = reinterpret_cast<LPVOID>(vlcConnectionPointContainer);
-    else if( IID_IObjectSafety == riid )
+    else if (IID_IObjectSafety == riid)
         *ppv = reinterpret_cast<LPVOID>(vlcObjectSafety);
-    else if( IID_IDispatch == riid )
-        *ppv = (CLSID_VLCPlugin2 == getClassID()) ?
-                reinterpret_cast<LPVOID>(vlcControl2) :
-                reinterpret_cast<LPVOID>(vlcControl);
-    else if( IID_IVLCControl == riid )
-        *ppv = reinterpret_cast<LPVOID>(vlcControl);
-    else if( IID_IVLCControl2 == riid )
+    else if( IID_IVLCControl2 == riid || IID_IDispatch == riid)
         *ppv = reinterpret_cast<LPVOID>(vlcControl2);
     else if( IID_IViewObject == riid )
         *ppv = reinterpret_cast<LPVOID>(vlcViewObject);
@@ -390,35 +371,30 @@ STDMETHODIMP_(ULONG) VLCPlugin::Release(void)
         return 0;
     }
     return _i_ref;
-};
+}
 
 //////////////////////////////////////
 
 HRESULT VLCPlugin::onInit(void)
 {
-    if( NULL == _p_libvlc )
-    {
-        // initialize persistable properties
-        set_autoplay(true);
-        _b_autoloop   = FALSE;
-        _bstr_baseurl = NULL;
-        _bstr_mrl     = NULL;
-        _b_visible    = TRUE;
-        _b_mute       = FALSE;
-        _i_volume     = 50;
-        _i_time       = 0;
-        _i_backcolor  = 0;
-        // set default/preferred size (320x240) pixels in HIMETRIC
-        HDC hDC = CreateDevDC(NULL);
-        _extent.cx = 320;
-        _extent.cy = 240;
-        HimetricFromDP(hDC, (LPPOINT)&_extent, 1);
-        DeleteDC(hDC);
-
-        return S_OK;
-    }
-    return CO_E_ALREADYINITIALIZED;
-};
+    // initialize persistable properties
+    set_autoplay(true);
+    _b_autoloop   = FALSE;
+    _bstr_baseurl = NULL;
+    _bstr_mrl     = NULL;
+    _b_visible    = TRUE;
+    _b_mute       = FALSE;
+    _i_volume     = 50;
+    _i_time       = 0;
+    _i_backcolor  = 0;
+    // set default/preferred size (320x240) pixels in HIMETRIC
+    HDC hDC = CreateDevDC(NULL);
+    _extent.cx = 320;
+    _extent.cy = 240;
+    HimetricFromDP(hDC, (LPPOINT)&_extent, 1);
+    DeleteDC(hDC);
+    return S_OK;
+}
 
 HRESULT VLCPlugin::onLoad(void)
 {
@@ -457,75 +433,31 @@ HRESULT VLCPlugin::onLoad(void)
     }
     setDirty(FALSE);
     return S_OK;
-};
+}
 
 void VLCPlugin::initVLC()
 {
-    static const char * const ppsz_argv[] = {
-        "-vv",
-        "--no-stats",
-        "--intf=dummy",
-        "--no-video-title-show",
-    };
+    try
+    {
+        static const char * const ppsz_argv[] = {
+            "-vv",
+            "--no-stats",
+            "--intf=dummy",
+            "--no-video-title-show",
+        };
 
-    _p_libvlc = libvlc_new(sizeof(ppsz_argv) / sizeof(*ppsz_argv), ppsz_argv);
-    if( !_p_libvlc )
+        auto instance = VLC::Instance( sizeof(ppsz_argv) / sizeof(*ppsz_argv), ppsz_argv );
+        if( !m_player.open( instance ) )
+            return;
+    }
+    catch (std::runtime_error&)
+    {
         return;
-
-    if( !vlc_player::open(_p_libvlc) )
-        return;
-
-    vlc_player::set_mode(_b_autoloop ? libvlc_playback_mode_loop :
-                                       libvlc_playback_mode_default);
+    }
 
     // register player events
     player_register_events();
 
-    // initial playlist item
-    if( SysStringLen(_bstr_mrl) > 0 )
-    {
-        char *psz_mrl = NULL;
-
-        if( SysStringLen(_bstr_baseurl) > 0 )
-        {
-            /*
-            ** if the MRL a relative URL, we should end up with an absolute URL
-            */
-            LPWSTR abs_url = CombineURL(_bstr_baseurl, _bstr_mrl);
-            if( NULL != abs_url )
-            {
-                psz_mrl = CStrFromWSTR(CP_UTF8, abs_url, wcslen(abs_url));
-                CoTaskMemFree(abs_url);
-            }
-            else
-            {
-                psz_mrl = CStrFromBSTR(CP_UTF8, _bstr_mrl);
-            }
-        }
-        else
-        {
-            /*
-            ** baseURL is empty, assume MRL is absolute
-            */
-            psz_mrl = CStrFromBSTR(CP_UTF8, _bstr_mrl);
-        }
-        if( NULL != psz_mrl )
-        {
-            const char *options[1];
-            int i_options = 0;
-
-            char timeBuffer[32];
-            if( _i_time )
-            {
-                snprintf(timeBuffer, sizeof(timeBuffer), ":start-time=%d", _i_time);
-                options[i_options++] = timeBuffer;
-            }
-            // add default target to playlist
-            playlist_add_extended_untrusted(psz_mrl, i_options, options);
-            CoTaskMemFree(psz_mrl);
-        }
-    }
-
     if( !isInPlaceActive()  )
     {
         LPOLECLIENTSITE pClientSite;
@@ -535,8 +467,6 @@ void VLCPlugin::initVLC()
             pClientSite->Release();
         }
     }
-
-    set_player_window();
 };
 
 void VLCPlugin::setErrorInfo(REFIID riid, const char *description)
@@ -630,7 +560,7 @@ HRESULT VLCPlugin::onAmbientChanged(LPUNKNOWN pContainer, DISPID dispID)
             break;
     }
     return S_OK;
-};
+}
 
 HRESULT VLCPlugin::onClose(DWORD)
 {
@@ -638,23 +568,14 @@ HRESULT VLCPlugin::onClose(DWORD)
     {
         onInPlaceDeactivate();
     }
-    if( isRunning() )
-    {
-        libvlc_instance_t* p_libvlc = _p_libvlc;
-
-        _p_libvlc = NULL;
-        vlcDataObject->onClose();
-
-        if( p_libvlc )
-            libvlc_release(p_libvlc);
-    }
+    vlcDataObject->onClose();
     return S_OK;
-};
+}
 
 BOOL VLCPlugin::isInPlaceActive(void)
 {
     return (NULL != _inplacewnd);
-};
+}
 
 HRESULT VLCPlugin::onActivateInPlace(LPMSG, HWND hwndParent, LPCRECT lprcPosRect, LPCRECT lprcClipRect)
 {
@@ -695,27 +616,66 @@ HRESULT VLCPlugin::onActivateInPlace(LPMSG, HWND hwndParent, LPCRECT lprcPosRect
     HRGN clipRgn = CreateRectRgnIndirect(&clipRect);
     SetWindowRgn(_inplacewnd, clipRgn, TRUE);
 
-    if( isUserMode() ) {
-        _WindowsManager.CreateWindows(this->getInPlaceWindow());
-    }
-
-    if( _b_usermode )
+    if( isUserMode() )
     {
-        /* will run vlc if not done already */
-        libvlc_instance_t* p_libvlc;
-        HRESULT result = getVLC(&p_libvlc);
-        if( FAILED(result) )
-            return result;
+        _WindowsManager.CreateWindows(this->getInPlaceWindow());
+
+        // initial playlist item
+        if( SysStringLen(_bstr_mrl) > 0 )
+        {
+            char *psz_mrl = NULL;
+
+            if( SysStringLen(_bstr_baseurl) > 0 )
+            {
+                /*
+                ** if the MRL a relative URL, we should end up with an absolute URL
+                */
+                LPWSTR abs_url = CombineURL(_bstr_baseurl, _bstr_mrl);
+                if( NULL != abs_url )
+                {
+                    psz_mrl = CStrFromWSTR(CP_UTF8, abs_url, wcslen(abs_url));
+                    CoTaskMemFree(abs_url);
+                }
+                else
+                {
+                    psz_mrl = CStrFromBSTR(CP_UTF8, _bstr_mrl);
+                }
+            }
+            else
+            {
+                /*
+                ** baseURL is empty, assume MRL is absolute
+                */
+                psz_mrl = CStrFromBSTR(CP_UTF8, _bstr_mrl);
+            }
+            if( NULL != psz_mrl )
+            {
+                const char *options[1];
+                int i_options = 0;
+
+                char timeBuffer[32];
+                if( _i_time )
+                {
+                    snprintf(timeBuffer, sizeof(timeBuffer), ":start-time=%d", _i_time);
+                    options[i_options++] = timeBuffer;
+                }
+                // add default target to playlist
+                m_player.add_item( psz_mrl, i_options, options);
+                CoTaskMemFree(psz_mrl);
+            }
+        }
 
         if( get_autoplay() )
         {
-            vlc_player::play(0);
+            get_player().play();
         }
     }
 
     if( isVisible() )
         ShowWindow(_inplacewnd, SW_SHOW);
 
+    set_player_window();
+
     return S_OK;
 };
 
@@ -724,13 +684,23 @@ void VLCPlugin::toggleFullscreen()
     _WindowsManager.ToggleFullScreen();
 }
 
+void VLCPlugin::setFullscreen(BOOL yes)
+{
+    if( yes )
+        _WindowsManager.StartFullScreen();
+    else
+        _WindowsManager.EndFullScreen();
+}
+
+BOOL VLCPlugin::getFullscreen()
+{
+    return _WindowsManager.IsFullScreen();
+}
+
 HRESULT VLCPlugin::onInPlaceDeactivate(void)
 {
-    if( isPlaying() )
-    {
-        playlist_stop();
-        fireOnStopEvent();
-    }
+    if( m_player.mlp().isPlaying() )
+        m_player.mlp().stop();
 
     _WindowsManager.DestroyWindows();
 
@@ -766,16 +736,10 @@ void VLCPlugin::setVolume(int volume)
     if( volume != _i_volume )
     {
         _i_volume = volume;
-        if( isRunning() )
-        {
-            libvlc_media_player_t *p_md;
-            HRESULT hr = getMD(&p_md);
-            if( SUCCEEDED(hr) )
-                libvlc_audio_set_volume(p_md, _i_volume);
-        }
-        setDirty(TRUE);
+        if ( m_player.get_mp().setVolume( volume ) )
+            setDirty(TRUE);
     }
-};
+}
 
 void VLCPlugin::setBackColor(OLE_COLOR backcolor)
 {
@@ -798,12 +762,18 @@ void VLCPlugin::setTime(int seconds)
     if( seconds != _i_time )
     {
         setStartTime(_i_time);
-        if( vlc_player::is_open() )
-        {
-            vlc_player::set_time(_i_time);
-        }
+        m_player.get_mp().setTime( _i_time );
     }
-};
+}
+
+void VLCPlugin::setMute(BOOL mute)
+{
+    if( mute != _b_mute )
+    {
+        _b_mute = mute;
+        m_player.get_mp().setMute( _b_mute );
+    }
+}
 
 void VLCPlugin::setFocus(BOOL fFocus)
 {
@@ -988,24 +958,6 @@ void VLCPlugin::firePropChangedEvent(DISPID dispid)
     vlcConnectionPointContainer->firePropChangedEvent(dispid);
 };
 
-void VLCPlugin::fireOnPlayEvent(void)
-{
-    DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};
-    vlcConnectionPointContainer->fireEvent(DISPID_PlayEvent, &dispparamsNoArgs);
-};
-
-void VLCPlugin::fireOnPauseEvent(void)
-{
-    DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};
-    vlcConnectionPointContainer->fireEvent(DISPID_PauseEvent, &dispparamsNoArgs);
-};
-
-void VLCPlugin::fireOnStopEvent(void)
-{
-    DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};
-    vlcConnectionPointContainer->fireEvent(DISPID_StopEvent, &dispparamsNoArgs);
-};
-
 /*
  * Async events
  */
@@ -1082,47 +1034,6 @@ void VLCPlugin::fireOnMediaPlayerBackwardEvent()
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerBackwardEvent, &dispparamsNoArgs);
 };
 
-static void handle_input_state_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    switch( event->type )
-    {
-        case libvlc_MediaPlayerMediaChanged:
-            plugin->fireOnMediaPlayerMediaChangedEvent();
-            break;        
-        case libvlc_MediaPlayerNothingSpecial:
-            plugin->fireOnMediaPlayerNothingSpecialEvent();
-            break;
-        case libvlc_MediaPlayerOpening:
-            plugin->fireOnMediaPlayerOpeningEvent();
-            break;
-        case libvlc_MediaPlayerBuffering:
-            plugin->fireOnMediaPlayerBufferingEvent(event->u.media_player_buffering.new_cache);
-            break;
-        case libvlc_MediaPlayerPlaying:
-            plugin->fireOnMediaPlayerPlayingEvent();
-            break;
-        case libvlc_MediaPlayerPaused:
-            plugin->fireOnMediaPlayerPausedEvent();
-            break;
-        case libvlc_MediaPlayerStopped:
-            plugin->fireOnMediaPlayerStoppedEvent();
-            break;
-        case libvlc_MediaPlayerForward:
-            plugin->fireOnMediaPlayerForwardEvent();
-            break;
-        case libvlc_MediaPlayerBackward:
-            plugin->fireOnMediaPlayerBackwardEvent();
-            break;
-        case libvlc_MediaPlayerEndReached:
-            plugin->fireOnMediaPlayerEndReachedEvent();
-            break;
-        case libvlc_MediaPlayerEncounteredError:
-            plugin->fireOnMediaPlayerEncounteredErrorEvent();
-            break;
-    }
-}
-
 void VLCPlugin::fireOnMediaPlayerTimeChangedEvent(libvlc_time_t  time)
 {
     DISPPARAMS params;
@@ -1136,12 +1047,6 @@ void VLCPlugin::fireOnMediaPlayerTimeChangedEvent(libvlc_time_t  time)
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerTimeChangedEvent, &params);
 };
 
-static void handle_time_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerTimeChangedEvent(event->u.media_player_time_changed.new_time);
-}
-
 void VLCPlugin::fireOnMediaPlayerPositionChangedEvent(float position)
 {
     DISPPARAMS params;
@@ -1155,13 +1060,6 @@ void VLCPlugin::fireOnMediaPlayerPositionChangedEvent(float position)
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerPositionChangedEvent, &params);
 };
 
-static void handle_position_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerPositionChangedEvent(event->u.media_player_position_changed.new_position);
-}
-
-#define B(val) ((val) ? 0xFFFF : 0x0000)
 void VLCPlugin::fireOnMediaPlayerSeekableChangedEvent(VARIANT_BOOL seekable)
 {
     DISPPARAMS params;
@@ -1173,12 +1071,6 @@ void VLCPlugin::fireOnMediaPlayerSeekableChangedEvent(VARIANT_BOOL seekable)
     params.rgdispidNamedArgs = NULL;
     params.cNamedArgs = 0;
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerSeekableChangedEvent, &params);
-};
-
-static void handle_seekable_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerSeekableChangedEvent(B(event->u.media_player_seekable_changed.new_seekable));
 }
 
 void VLCPlugin::fireOnMediaPlayerPausableChangedEvent(VARIANT_BOOL pausable)
@@ -1194,13 +1086,6 @@ void VLCPlugin::fireOnMediaPlayerPausableChangedEvent(VARIANT_BOOL pausable)
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerPausableChangedEvent, &params);
 };
 
-static void handle_pausable_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerPausableChangedEvent(B(event->u.media_player_pausable_changed.new_pausable));
-}
-#undef B
-
 void VLCPlugin::fireOnMediaPlayerTitleChangedEvent(int title)
 {
     DISPPARAMS params;
@@ -1214,12 +1099,6 @@ void VLCPlugin::fireOnMediaPlayerTitleChangedEvent(int title)
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerTitleChangedEvent, &params);
 };
 
-static void handle_title_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerTitleChangedEvent(event->u.media_player_title_changed.new_title);
-}
-
 void VLCPlugin::fireOnMediaPlayerLengthChangedEvent(long length)
 {
     DISPPARAMS params;
@@ -1231,89 +1110,72 @@ void VLCPlugin::fireOnMediaPlayerLengthChangedEvent(long length)
     params.rgdispidNamedArgs = NULL;
     params.cNamedArgs = 0;
     vlcConnectionPointContainer->fireEvent(DISPID_MediaPlayerLengthChangedEvent, &params);
-};
-
-static void handle_length_changed_event(const libvlc_event_t* event, void *param)
-{
-    VLCPlugin *plugin = (VLCPlugin*)param;
-    plugin->fireOnMediaPlayerLengthChangedEvent(event->u.media_player_length_changed.new_length);
 }
 
 /* */
 
 void VLCPlugin::set_player_window()
 {
-    _WindowsManager.LibVlcAttach( &get_player() );
+    if (_WindowsManager.getHolderWnd())
+        m_player.get_mp().setHwnd( _WindowsManager.getHolderWnd()->hWnd() );
 }
 
-void VLCPlugin::on_player_action(vlc_player_action_e pa)
-{
-    switch(pa){
-    case pa_play:
-        fireOnPlayEvent();
-        break;
-    case pa_pause:
-        fireOnPauseEvent();
-        break;
-    case pa_stop:
-        fireOnStopEvent();
-        break;
-    case pa_next:
-    case pa_prev:
-        break;
-    }
-}
-
-static vlcplugin_event_t vlcevents[] = {
-    { libvlc_MediaPlayerMediaChanged, handle_input_state_event },
-    { libvlc_MediaPlayerNothingSpecial, handle_input_state_event },
-    { libvlc_MediaPlayerOpening, handle_input_state_event },
-    { libvlc_MediaPlayerBuffering, handle_input_state_event },
-    { libvlc_MediaPlayerPlaying, handle_input_state_event },
-    { libvlc_MediaPlayerPaused, handle_input_state_event },
-    { libvlc_MediaPlayerStopped, handle_input_state_event },
-    { libvlc_MediaPlayerForward, handle_input_state_event },
-    { libvlc_MediaPlayerBackward, handle_input_state_event },
-    { libvlc_MediaPlayerEndReached, handle_input_state_event },
-    { libvlc_MediaPlayerEncounteredError, handle_input_state_event },
-    { libvlc_MediaPlayerTimeChanged, handle_time_changed_event },
-    { libvlc_MediaPlayerPositionChanged, handle_position_changed_event },
-    { libvlc_MediaPlayerSeekableChanged, handle_seekable_changed_event },
-    { libvlc_MediaPlayerPausableChanged, handle_pausable_changed_event },
-    { libvlc_MediaPlayerTitleChanged, handle_title_changed_event },
-    { libvlc_MediaPlayerLengthChanged, handle_length_changed_event },
-};
+#define B(val) ((val) ? 0xFFFF : 0x0000)
 
 void VLCPlugin::player_register_events()
 {
-    libvlc_event_manager_t *eventManager = NULL;
-    assert(vlc_player::is_open());
-
-    eventManager = libvlc_media_player_event_manager(vlc_player::get_mp());
-    if(eventManager) {
-        /* attach all libvlc events we care about */
-        for( size_t i = 0; i < ARRAY_SIZE(vlcevents); i++ )
-        {
-            libvlc_event_attach( eventManager, vlcevents[i].libvlc_type,
-                                               vlcevents[i].libvlc_callback,
-                                               this );
-        }
-    }
+    auto& em = m_player.get_mp().eventManager();
+    em.onMediaChanged([this](VLC::MediaPtr) {
+        fireOnMediaPlayerMediaChangedEvent();
+    });
+    em.onNothingSpecial([this] {
+        fireOnMediaPlayerNothingSpecialEvent();
+    });
+    em.onOpening([this] {
+        fireOnMediaPlayerOpeningEvent();
+    });
+    em.onBuffering([this](float b) {
+        fireOnMediaPlayerBufferingEvent(b);
+    });
+    em.onPlaying([this] {
+        fireOnMediaPlayerPlayingEvent();
+    });
+    em.onPaused([this] {
+        fireOnMediaPlayerPausedEvent();
+    });
+    em.onStopped([this] {
+        fireOnMediaPlayerStoppedEvent();
+    });
+    em.onForward([this] {
+        fireOnMediaPlayerForwardEvent();
+    });
+    em.onBackward([this] {
+        fireOnMediaPlayerBackwardEvent();
+    });
+    em.onEndReached([this] {
+        fireOnMediaPlayerEndReachedEvent();
+    });
+    em.onEncounteredError([this] {
+        fireOnMediaPlayerEncounteredErrorEvent();
+    });
+    em.onTimeChanged([this] (int64_t time) {
+        fireOnMediaPlayerTimeChangedEvent( time );
+    });
+    em.onPositionChanged([this](float pos) {
+        fireOnMediaPlayerPositionChangedEvent( pos );
+    });
+    em.onSeekableChanged([this](bool b) {
+        fireOnMediaPlayerSeekableChangedEvent( B( b ) );
+    });
+    em.onPausableChanged([this](bool b) {
+        fireOnMediaPlayerPausableChangedEvent( B( b ) );
+    });
+    em.onTitleChanged([this](int t) {
+        fireOnMediaPlayerTitleChangedEvent( t );
+    });
+    em.onLengthChanged( [this]( int64_t length ) {
+        fireOnMediaPlayerLengthChangedEvent( length );
+    });
 }
 
-void VLCPlugin::player_unregister_events()
-{
-    libvlc_event_manager_t *eventManager = NULL;
-    assert(vlc_player::is_open());
-
-    eventManager = libvlc_media_player_event_manager(vlc_player::get_mp());
-    if(eventManager) {
-        /* detach all libvlc events we cared about */
-        for( size_t i = 0; i < ARRAY_SIZE(vlcevents); i++ )
-        {
-            libvlc_event_detach( eventManager, vlcevents[i].libvlc_type,
-                                               vlcevents[i].libvlc_callback,
-                                               this );
-        }
-    }
-}
+#undef B
