@@ -34,23 +34,20 @@ public:
     VLCInterfaceBase(VLCPlugin *p): _plug(p), _ti(NULL) { }
     virtual ~VLCInterfaceBase() { if( _ti ) _ti->Release(); }
 
-    VLCPlugin *Instance() const { return _plug; }
-    HRESULT getVLC(libvlc_instance_t **pp) const { return _plug->getVLC(pp); }
-    HRESULT getMD(libvlc_media_player_t **pp) const { return _plug->getMD(pp); }
-
 protected:
     HRESULT loadTypeInfo(REFIID riid);
     ITypeInfo *TypeInfo() const { return _ti; }
 
     STDMETHODIMP_(ULONG) AddRef(void) { return _plug->pUnkOuter->AddRef(); };
     STDMETHODIMP_(ULONG) Release(void) { return _plug->pUnkOuter->Release(); };
-private:
+
     VLCPlugin *_plug;
+private:
     ITypeInfo *_ti;
 };
 
 template<class T,class I>
-class VLCInterface: public I, private VLCInterfaceBase
+class VLCInterface: public I, protected VLCInterfaceBase
 {
 private:
     typedef VLCInterfaceBase Base;
@@ -64,12 +61,6 @@ private:
 
 public:
     VLCInterface(VLCPlugin *p): Base(p) { }
-    VLCPlugin *Instance() const { return Base::Instance(); }
-
-    virtual ~VLCInterface() { }
-
-    HRESULT getVLC(libvlc_instance_t **pp) const { return Base::getVLC(pp); }
-    HRESULT getMD(libvlc_media_player_t **pp) const { return Base::getMD(pp); }
 
     STDMETHODIMP QueryInterface(REFIID riid, void **ppv)
     {
@@ -146,13 +137,45 @@ public:
     STDMETHODIMP get_channel(long*);
     STDMETHODIMP put_channel(long);
     STDMETHODIMP toggleMute();
+    STDMETHODIMP description(long trackId, BSTR*);
+};
+
+class VLCTitle : public VLCInterface<VLCTitle,IVLCTitle>
+{
+public:
+    VLCTitle(VLCPlugin *p): VLCInterface<VLCTitle,IVLCTitle>(p) { }
+
+    // IVLCTitle methods
+    STDMETHODIMP get_count(long*);
+    STDMETHODIMP get_track(long*);
+    STDMETHODIMP put_track(long);
     STDMETHODIMP description(long, BSTR*);
 };
 
+class VLCChapter : public VLCInterface<VLCChapter,IVLCChapter>
+{
+public:
+    VLCChapter(VLCPlugin *p): VLCInterface<VLCChapter,IVLCChapter>(p) { }
+
+    // IVLCChapter methods
+    STDMETHODIMP get_count(long*);
+    STDMETHODIMP countForTitle(long, long*);
+    STDMETHODIMP get_track(long*);
+    STDMETHODIMP put_track(long);
+    STDMETHODIMP description(long, long, BSTR*);
+    STDMETHODIMP next();
+    STDMETHODIMP prev();
+};
+
 class VLCInput: public VLCInterface<VLCInput,IVLCInput>
 {
 public:
-    VLCInput(VLCPlugin *p): VLCInterface<VLCInput,IVLCInput>(p) { }
+    VLCInput(VLCPlugin *p): VLCInterface<VLCInput,IVLCInput>(p),
+        _p_vlctitle(new VLCTitle(p)), _p_vlcchapter(new VLCChapter(p)) { }
+    virtual ~VLCInput() {
+        _p_vlctitle->Release();
+        _p_vlcchapter->Release();
+    }
 
     // IVLCInput methods
     STDMETHODIMP get_length(double*);
@@ -165,6 +188,12 @@ public:
     STDMETHODIMP put_rate(double);
     STDMETHODIMP get_fps(double*);
     STDMETHODIMP get_hasVout(VARIANT_BOOL*);
+    STDMETHODIMP get_title(IVLCTitle**);
+    STDMETHODIMP get_chapter(IVLCChapter**);
+
+private:
+    IVLCTitle       *_p_vlctitle;
+    IVLCChapter     *_p_vlcchapter;
 };
 
 class VLCMarquee: public VLCInterface<VLCMarquee,IVLCMarquee>
@@ -373,7 +402,7 @@ public:
     virtual ~VLCControl2();
 
     // IUnknown methods
-    STDMETHODIMP QueryInterface(REFIID riid, void **ppv)
+    STDMETHODIMP QueryInterface(REFIID riid, void **ppv) override
     {
         if( NULL == ppv )
           return E_POINTER;
@@ -388,45 +417,47 @@ public:
         return _p_instance->pUnkOuter->QueryInterface(riid, ppv);
     };
 
-    STDMETHODIMP_(ULONG) AddRef(void) { return _p_instance->pUnkOuter->AddRef(); };
-    STDMETHODIMP_(ULONG) Release(void) { return _p_instance->pUnkOuter->Release(); };
+    STDMETHODIMP_(ULONG) AddRef(void) override { return _p_instance->pUnkOuter->AddRef(); }
+    STDMETHODIMP_(ULONG) Release(void) override { return _p_instance->pUnkOuter->Release(); }
 
     // IDispatch methods
-    STDMETHODIMP GetTypeInfoCount(UINT*);
-    STDMETHODIMP GetTypeInfo(UINT, LCID, LPTYPEINFO*);
-    STDMETHODIMP GetIDsOfNames(REFIID,LPOLESTR*,UINT,LCID,DISPID*);
-    STDMETHODIMP Invoke(DISPID,REFIID,LCID,WORD,DISPPARAMS*,VARIANT*,EXCEPINFO*,UINT*);
+    STDMETHODIMP GetTypeInfoCount(UINT*) override;
+    STDMETHODIMP GetTypeInfo(UINT, LCID, LPTYPEINFO*) override;
+    STDMETHODIMP GetIDsOfNames(REFIID, LPOLESTR*, UINT, LCID, DISPID*) override;
+    STDMETHODIMP Invoke(DISPID, REFIID, LCID, WORD, DISPPARAMS*, VARIANT*, EXCEPINFO*, UINT*) override;
 
     // IVLCControl2 methods
-    STDMETHODIMP get_AutoLoop(VARIANT_BOOL *autoloop);
-    STDMETHODIMP put_AutoLoop(VARIANT_BOOL autoloop);
-    STDMETHODIMP get_AutoPlay(VARIANT_BOOL *autoplay);
-    STDMETHODIMP put_AutoPlay(VARIANT_BOOL autoplay);
-    STDMETHODIMP get_BaseURL(BSTR *url);
-    STDMETHODIMP put_BaseURL(BSTR url);
-    STDMETHODIMP get_MRL(BSTR *mrl);
-    STDMETHODIMP put_MRL(BSTR mrl);
-    STDMETHODIMP get_Toolbar(VARIANT_BOOL *visible);
-    STDMETHODIMP put_Toolbar(VARIANT_BOOL visible);
-    STDMETHODIMP get_StartTime(long *seconds);
-    STDMETHODIMP put_StartTime(long seconds);
-    STDMETHODIMP get_VersionInfo(BSTR *version);
-    STDMETHODIMP versionInfo(BSTR *version);
-    STDMETHODIMP get_Visible(VARIANT_BOOL *visible);
-    STDMETHODIMP put_Visible(VARIANT_BOOL visible);
-    STDMETHODIMP get_Volume(long *volume);
-    STDMETHODIMP put_Volume(long volume);
-    STDMETHODIMP get_BackColor(OLE_COLOR *backcolor);
-    STDMETHODIMP put_BackColor(OLE_COLOR backcolor);
-    STDMETHODIMP get_FullscreenEnabled(VARIANT_BOOL* enabled);
-    STDMETHODIMP put_FullscreenEnabled(VARIANT_BOOL enabled);
+    STDMETHODIMP get_AutoLoop(VARIANT_BOOL *autoloop) override;
+    STDMETHODIMP put_AutoLoop(VARIANT_BOOL autoloop) override;
+    STDMETHODIMP get_AutoPlay(VARIANT_BOOL *autoplay) override;
+    STDMETHODIMP put_AutoPlay(VARIANT_BOOL autoplay) override;
+    STDMETHODIMP get_BaseURL(BSTR *url) override;
+    STDMETHODIMP put_BaseURL(BSTR url) override;
+    STDMETHODIMP get_MRL(BSTR *mrl) override;
+    STDMETHODIMP put_MRL(BSTR mrl) override;
+    STDMETHODIMP get_Toolbar(VARIANT_BOOL *visible) override;
+    STDMETHODIMP put_Toolbar(VARIANT_BOOL visible) override;
+    STDMETHODIMP get_StartTime(long *seconds) override;
+    STDMETHODIMP put_StartTime(long seconds) override;
+    STDMETHODIMP get_VersionInfo(BSTR *version) override;
+    STDMETHODIMP getVersionInfo(BSTR *version) override;
+    STDMETHODIMP get_Visible(VARIANT_BOOL *visible) override;
+    STDMETHODIMP put_Visible(VARIANT_BOOL visible) override;
+    STDMETHODIMP get_Volume(long *volume) override;
+    STDMETHODIMP put_Volume(long volume) override;
+    STDMETHODIMP get_BackColor(OLE_COLOR *backcolor) override;
+    STDMETHODIMP put_BackColor(OLE_COLOR backcolor) override;
+    STDMETHODIMP get_FullscreenEnabled(VARIANT_BOOL* enabled) override;
+    STDMETHODIMP put_FullscreenEnabled(VARIANT_BOOL enabled) override;
+    STDMETHODIMP get_Branding(VARIANT_BOOL* visible) override;
+    STDMETHODIMP put_Branding(VARIANT_BOOL visible) override;
 
-    STDMETHODIMP get_audio(IVLCAudio**);
-    STDMETHODIMP get_input(IVLCInput**);
-    STDMETHODIMP get_playlist(IVLCPlaylist**);
-    STDMETHODIMP get_subtitle(IVLCSubtitle**);
-    STDMETHODIMP get_video(IVLCVideo**);
-    STDMETHODIMP get_mediaDescription(IVLCMediaDescription**);
+    STDMETHODIMP get_audio(IVLCAudio**) override;
+    STDMETHODIMP get_input(IVLCInput**) override;
+    STDMETHODIMP get_playlist(IVLCPlaylist**) override;
+    STDMETHODIMP get_subtitle(IVLCSubtitle**) override;
+    STDMETHODIMP get_video(IVLCVideo**) override;
+    STDMETHODIMP get_mediaDescription(IVLCMediaDescription**) override;
 
 protected:
     HRESULT loadTypeInfo();
