@@ -21,7 +21,7 @@
 
 #pragma once
 
-#include <vlc/vlc.h>
+#include <vlcpp/vlc.hpp>
 
 enum vlc_player_action_e
 {
@@ -35,20 +35,11 @@ enum vlc_player_action_e
 class vlc_player
 {
 public:
-    vlc_player();
-    ~vlc_player(void);
-
-    bool open(libvlc_instance_t* inst);
-    void close();
-
-    bool is_open() const { return _ml_p != 0; }
-    bool is_playing();
-    libvlc_state_t get_state();
-    bool is_stopped() { return libvlc_Stopped == get_state(); }
+    bool open(VLC::Instance& inst);
 
     int add_item(const char * mrl, unsigned int optc, const char **optv);
     int add_item(const char * mrl)
-        { return add_item(mrl, 0, 0); }
+        { return add_item( mrl, 0, nullptr ); }
 
     int  current_item();
     int  items_count();
@@ -56,54 +47,28 @@ public:
     void clear_items();
 
     void play();
-    bool play(unsigned int idx);
-    void pause();
-    void togglePause();
-    void stop();
 
-    bool next();
-    bool prev();
+    VLC::MediaPlayer& get_mp()
+    {
+        return _mp;
+    }
 
-    float get_rate();
-    void set_rate(float);
+    VLC::MediaListPlayer& mlp()
+    {
+        return _ml_p;
+    }
 
-    float get_fps();
-
-    bool has_vout();
-
-    float get_position();
-    void  set_position(float);
-
-    libvlc_time_t get_time();
-    void set_time(libvlc_time_t);
-
-    libvlc_time_t get_length();
-
-    void set_mode(libvlc_playback_mode_t);
-
-    bool is_muted();
-    void toggle_mute();
-    void set_mute(bool);
-
-    unsigned int get_volume();
-    void set_volume(unsigned int);
-
-    unsigned int track_count();
-    unsigned int get_track();
-    void set_track(unsigned int);
-
-    unsigned int get_channel();
-    void set_channel(unsigned int);
-
-    libvlc_media_player_t* get_mp() const
-        { return _mp; }
-
-protected:
-    virtual void on_player_action( vlc_player_action_e ){};
+    int currentAudioTrack();
+    int currentSubtitleTrack();
 
 private:
-    libvlc_instance_t *         _libvlc_instance;
-    libvlc_media_player_t*      _mp;
-    libvlc_media_list_t*        _ml;
-    libvlc_media_list_player_t* _ml_p;
+    // Returns a 0-based track index, instead of the internal libvlc one
+    int getTrack( int currentId, const std::vector<VLC::TrackDescription>& tracks );
+
+
+private:
+    VLC::Instance           _libvlc_instance;
+    VLC::MediaPlayer        _mp;
+    VLC::MediaList          _ml;
+    VLC::MediaListPlayer    _ml_p;
 };
