@@ -25,6 +25,7 @@
 #endif
 
 #include <windows.h>
+#include <windowsx.h>
 #include <commctrl.h>
 #include <uxtheme.h>
 
@@ -144,7 +145,7 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                              ControlWidth, ControlsHeight, hWnd(),
                              (HMENU)ID_FS_PLAY_PAUSE, 0, 0);
             SendMessage(hPlayPauseButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP, (LPARAM)RC().hPauseBitmap);
+                        (WPARAM)IMAGE_BITMAP, (LPARAM)RC().hPlayBitmap);
             HorizontalOffset+=ControlWidth+xControlsSpace;
 
             ControlWidth = 200;
@@ -165,6 +166,7 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                 FreeLibrary(hThModule);
             }
             HorizontalOffset+=ControlWidth+xControlsSpace;
+            SendMessage(hVideoPosScroll, (UINT)PBM_SETRANGE, 0, MAKELPARAM(0, 1000));
 
             ControlWidth = ButtonsWidth;
             hMuteButton =
@@ -200,7 +202,7 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                              ControlWidth, ControlsHeight, hWnd(),
                              (HMENU)ID_FS_SWITCH_FS, 0, 0);
             SendMessage(hFSButton, BM_SETIMAGE, (WPARAM)IMAGE_BITMAP,
-                        (LPARAM)RC().hDeFullscreenBitmap);
+                        (LPARAM)RC().hFullscreenBitmap);
             HorizontalOffset+=ControlWidth+xControlsSpace;
 
             RECT rect;
@@ -213,17 +215,14 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                          rect.right-rect.left, ControlWndHeight,
                          SWP_NOZORDER|SWP_NOOWNERZORDER|SWP_NOACTIVATE);
 
-            //new message blinking timer
-            SetTimer(hWnd(), 2, 500, NULL);
-
             CreateToolTip();
 
-            break;
-        }
-        case WM_SHOWWINDOW:{
-            if(FALSE!=wParam){ //showing
-                UpdateButtons();
+            if( VP() ){
+                RegisterToVLCEvents();
+                UpdateVolumeSlider( VP()->get_mp().volume() );
+                UpdateMuteButton( VP()->get_mp().mute() );
             }
+
             break;
         }
         case WM_LBUTTONUP:{
@@ -237,25 +236,16 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
             break;
         }
         case WM_TIMER:{
-            switch(wParam){
-                case 1:{
-                    POINT MousePoint;
-                    GetCursorPos(&MousePoint);
-                    RECT ControlWndRect;
-                    GetWindowRect(hWnd(), &ControlWndRect);
-                    if(PtInRect(&ControlWndRect, MousePoint)||GetCapture()==hVolumeSlider){
-                        //do not allow control window to close while mouse is within
-                        NeedShowControls();
-                    }
-                    else{
-                        NeedHideControls();
-                    }
-                    break;
-                }
-                case 2:{
-                    UpdateButtons();
-                    break;
-                }
+            POINT MousePoint;
+            GetCursorPos(&MousePoint);
+            RECT ControlWndRect;
+            GetWindowRect(hWnd(), &ControlWndRect);
+            if(PtInRect(&ControlWndRect, MousePoint)||GetCapture()==hVolumeSlider){
+                //do not allow control window to close while mouse is within
+                NeedShowControls();
+            }
+            else{
+                NeedHideControls();
             }
             break;
         }
@@ -287,7 +277,7 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                         case ID_FS_PLAY_PAUSE:{
                             if( VP() ){
                                 if( IsPlaying() )
-                                    VP()->pause();
+                                    VP()->mlp().pause();
                                 else
                                     VP()->play();
                             }
@@ -295,8 +285,9 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
                         }
                         case ID_FS_MUTE:{
                             if( VP() ){
-                                VP()->set_mute( IsDlgButtonChecked(hWnd(), ID_FS_MUTE) != FALSE );
-                                SyncVolumeSliderWithVLCVolume();
+                                bool newMutedState = IsDlgButtonChecked(hWnd(), ID_FS_MUTE) != FALSE;
+                                VP()->get_mp().setMute( newMutedState );
+                                UpdateMuteButton( newMutedState );
                             }
                             break;
                         }
@@ -395,10 +386,10 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
         }
         case WM_HSCROLL:
         case WM_VSCROLL: {
-            if( VP() ){
-                if(hVolumeSlider==(HWND)lParam){
+            if( hVolumeSlider == (HWND)lParam ){
+                if( VP() ){
                     LRESULT SliderPos = SendMessage(hVolumeSlider, (UINT) TBM_GETPOS, 0, 0);
-                    SetVLCVolumeBySliderPos(SliderPos);
+                    VP()->get_mp().setVolume( SliderPos );
                 }
             }
             break;
@@ -409,49 +400,43 @@ LRESULT VLCControlsWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
     return 0L;
 }
 
-void VLCControlsWnd::UpdateButtons()
+void VLCControlsWnd::RegisterToVLCEvents()
 {
-    if(hVideoPosScroll){
-        SetVideoPosScrollRangeByVideoLen();
-        SyncVideoPosScrollPosWithVideoPos();
-    }
+    VP()->get_mp().eventManager().onPositionChanged([this](float pos) {
+        PostMessage(hVideoPosScroll, (UINT) PBM_SETPOS, (WPARAM)(pos * 1000), 0);
+    });
 
-    if(hVolumeSlider){
-        SyncVolumeSliderWithVLCVolume();
-    }
+    VP()->get_mp().eventManager().onPlaying([this] {
+        PostMessage(hPlayPauseButton, BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) RC().hPauseBitmap);
+    });
 
-    if(hFSButton) {
-        HANDLE hFSBitmap =
-            WM().IsFullScreen() ? RC().hDeFullscreenBitmap :
-                                  RC().hFullscreenBitmap;
+    VP()->get_mp().eventManager().onPaused([this] {
+        PostMessage(hPlayPauseButton, BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) RC().hPlayBitmap);
+    });
 
-        HANDLE hBitmap =
-            (HANDLE)SendMessage(hFSButton, BM_GETIMAGE, (WPARAM)IMAGE_BITMAP, 0);
+    VP()->get_mp().eventManager().onStopped([this] {
+        PostMessage(hPlayPauseButton, BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) RC().hPlayBitmap);
+        PostMessage(hVideoPosScroll, (UINT) PBM_SETPOS, (WPARAM)0, 0);
+    });
 
-        if( WM().getNewMessageFlag() &&
-           (hBitmap == RC().hDeFullscreenBitmap ||
-            hBitmap == RC().hFullscreenBitmap) )
-        {
-            SendMessage(hFSButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP,
-                        (LPARAM)RC().hNewMessageBitmap);
-            //do not allow control window to close while there are new messages
-            NeedShowControls();
-        }
-        else{
-            if(hBitmap != hFSBitmap)
-                SendMessage(hFSButton, BM_SETIMAGE,
-                            (WPARAM)IMAGE_BITMAP,
-                            (LPARAM)hFSBitmap);
-        }
-    }
+    VP()->get_mp().eventManager().onEndReached([this] {
+        PostMessage(hPlayPauseButton, BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) RC().hPlayBitmap);
+        PostMessage(hVideoPosScroll, (UINT) PBM_SETPOS, (WPARAM)0, 0);
+    });
 
-    if(hPlayPauseButton){
-        HANDLE hBmp = IsPlaying() ? RC().hPauseBitmap : RC().hPlayBitmap;
-        SendMessage(hPlayPauseButton, BM_SETIMAGE,
-                    (WPARAM)IMAGE_BITMAP, (LPARAM)hBmp);
-    }
+#if LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 2, 2, 0)
+    VP()->get_mp().eventManager().onAudioVolume([this](float vol) {
+        UpdateVolumeSlider( roundf(vol * 100) );
+    });
 
+    VP()->get_mp().eventManager().onMuted([this] {
+        UpdateMuteButton(true);
+    });
+
+    VP()->get_mp().eventManager().onUnmuted([this] {
+        UpdateMuteButton(false);
+    });
+#endif
 }
 
 void VLCControlsWnd::NeedShowControls()
@@ -460,139 +445,59 @@ void VLCControlsWnd::NeedShowControls()
         if(WM().IsFullScreen() || (PO() && PO()->get_show_toolbar() ) )
             ShowWindow( hWnd(), SW_SHOW );
     }
-    //hide controls after 2 seconds
-    SetTimer(hWnd(), 1, 2*1000, NULL);
+    SetTimer(hWnd(), 1, 2 * 1000, nullptr);
 }
 
 void VLCControlsWnd::NeedHideControls()
 {
-    KillTimer( hWnd(), 1 );
     ShowWindow( hWnd(), SW_HIDE );
-}
-
-void VLCControlsWnd::SyncVideoPosScrollPosWithVideoPos()
-{
-    if( VP() ){
-        libvlc_time_t pos = VP()->get_time();
-        SetVideoPosScrollPosByVideoPos(pos);
-    }
-}
-
-void VLCControlsWnd::SetVideoPosScrollRangeByVideoLen()
-{
-    if( VP() ){
-        libvlc_time_t MaxLen = VP()->get_length();
-        VideoPosShiftBits = 0;
-        while(MaxLen>0xffff){
-            MaxLen >>= 1;
-            ++VideoPosShiftBits;
-        };
-        SendMessage(hVideoPosScroll, (UINT)PBM_SETRANGE, 0, MAKELPARAM(0, MaxLen));
-    }
-}
-
-void VLCControlsWnd::SetVideoPosScrollPosByVideoPos(libvlc_time_t CurScrollPos)
-{
-    PostMessage(hVideoPosScroll, (UINT)PBM_SETPOS, (WPARAM) (CurScrollPos >> VideoPosShiftBits), 0);
+    KillTimer(hWnd(), 1);
 }
 
 void VLCControlsWnd::SetVideoPos(float Pos) //0-start, 1-end
 {
     if( VP() ){
-        vlc_player& vp = *VP();
-        vp.set_time( static_cast<libvlc_time_t>( vp.get_length()*Pos ) );
-        SyncVideoPosScrollPosWithVideoPos();
+        VP()->get_mp().setPosition( Pos );
+
+        if( VP()->get_mp().length() > 0 )
+            PostMessage(hVideoPosScroll, (UINT)PBM_SETPOS, (WPARAM) (Pos * 1000), 0);
     }
 }
 
-void VLCControlsWnd::SyncVolumeSliderWithVLCVolume()
+void VLCControlsWnd::UpdateVolumeSlider(unsigned int vol)
 {
-    if( VP() ){
-        vlc_player& vp = *VP();
-        unsigned int vol = vp.get_volume();
-        const LRESULT SliderPos = SendMessage(hVolumeSlider, (UINT) TBM_GETPOS, 0, 0);
-        if((UINT)SliderPos!=vol)
-            SendMessage(hVolumeSlider, (UINT) TBM_SETPOS, (WPARAM) TRUE, (LPARAM) vol);
+    const LRESULT SliderPos = SendMessage(hVolumeSlider, (UINT) TBM_GETPOS, 0, 0);
+    if( (UINT)SliderPos != vol )
+        PostMessage(hVolumeSlider, (UINT) TBM_SETPOS, (WPARAM) TRUE, (LPARAM) vol);
+}
 
-        bool muted = vp.is_muted();
-        int MuteButtonState = SendMessage(hMuteButton, (UINT) BM_GETCHECK, 0, 0);
-        if((muted&&(BST_UNCHECKED==MuteButtonState))||(!muted&&(BST_CHECKED==MuteButtonState))){
-            SendMessage(hMuteButton, BM_SETCHECK, (WPARAM)(muted?BST_CHECKED:BST_UNCHECKED), 0);
-        }
-        LRESULT lResult = SendMessage(hMuteButton, BM_GETIMAGE, (WPARAM)IMAGE_BITMAP, 0);
-        if( (muted && ((HANDLE)lResult == RC().hVolumeBitmap)) ||
-            (!muted&&((HANDLE)lResult == RC().hVolumeMutedBitmap)) )
-        {
-            HANDLE hBmp = muted ? RC().hVolumeMutedBitmap : RC().hVolumeBitmap ;
-            SendMessage(hMuteButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP, (LPARAM)hBmp);
-        }
+void VLCControlsWnd::UpdateMuteButton(bool muted)
+{
+    int MuteButtonState = SendMessage(hMuteButton, (UINT) BM_GETCHECK, 0, 0);
+    if((muted&&(BST_UNCHECKED==MuteButtonState))||(!muted&&(BST_CHECKED==MuteButtonState))){
+        PostMessage(hMuteButton, BM_SETCHECK, (WPARAM)(muted?BST_CHECKED:BST_UNCHECKED), 0);
     }
-}
-
-void VLCControlsWnd::SetVLCVolumeBySliderPos(int CurPos)
-{
-    if( VP() ){
-        vlc_player& vp = *VP();
-        vp.set_volume(CurPos);
-        if(0==CurPos){
-            vp.set_mute( IsDlgButtonChecked( hWnd(), ID_FS_MUTE) != FALSE );
-        }
-        SyncVolumeSliderWithVLCVolume();
-    }
-}
-
-
-/////////////////////////////////////
-//VLCControlsWnd event handlers
-
-void VLCControlsWnd::handle_position_changed_event(const libvlc_event_t* )
-{
-    SyncVideoPosScrollPosWithVideoPos();
-}
-
-void VLCControlsWnd::handle_input_state_event(const libvlc_event_t* event)
-{
-    switch( event->type )
+    LRESULT lResult = SendMessage(hMuteButton, BM_GETIMAGE, (WPARAM)IMAGE_BITMAP, 0);
+    if( (muted && ((HANDLE)lResult == RC().hVolumeBitmap)) ||
+        (!muted&&((HANDLE)lResult == RC().hVolumeMutedBitmap)) )
     {
-        case libvlc_MediaPlayerPlaying:
-            PostMessage(hPlayPauseButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP, (LPARAM)RC().hPauseBitmap);
-            break;
-        case libvlc_MediaPlayerPaused:
-            PostMessage(hPlayPauseButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP, (LPARAM)RC().hPlayBitmap);
-            break;
-        case libvlc_MediaPlayerStopped:
-            PostMessage(hPlayPauseButton, BM_SETIMAGE,
-                        (WPARAM)IMAGE_BITMAP, (LPARAM)RC().hPlayBitmap);
-            break;
+        HANDLE hBmp = muted ? RC().hVolumeMutedBitmap : RC().hVolumeBitmap ;
+        PostMessage(hMuteButton, BM_SETIMAGE,
+             (WPARAM)IMAGE_BITMAP, (LPARAM)hBmp);
     }
 }
 
-//libvlc events arrives from separate thread
-void VLCControlsWnd::OnLibVlcEvent(const libvlc_event_t* event)
+void VLCControlsWnd::UpdateFullscreenButton(bool fullscreen)
 {
-    switch(event->type){
-        case libvlc_MediaPlayerPlaying:
-        case libvlc_MediaPlayerPaused:
-        case libvlc_MediaPlayerStopped:
-            handle_input_state_event(event);
-            break;
-        case libvlc_MediaPlayerPositionChanged:
-            handle_position_changed_event(event);
-            break;
-    }
+    if (fullscreen)
+        PostMessage( hFSButton, BM_SETIMAGE, (WPARAM)IMAGE_BITMAP, (LPARAM) RC().hDeFullscreenBitmap );
+    else
+        PostMessage( hFSButton, BM_SETIMAGE, (WPARAM)IMAGE_BITMAP, (LPARAM) RC().hFullscreenBitmap );
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 //VLCHolderWnd members
 ////////////////////////////////////////////////////////////////////////////////
-enum{
-    WM_TRY_SET_MOUSE_HOOK = WM_USER+1,
-    WM_MOUSE_EVENT_NOTIFY = WM_APP+1,
-    WM_MOUSE_EVENT_NOTIFY_SUCCESS = 0xFF
-};
 
 VLCHolderWnd*
 VLCHolderWnd::CreateHolderWindow(HINSTANCE hInstance,
@@ -648,29 +553,42 @@ LRESULT VLCHolderWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
             _CtrlsWnd =
                 VLCControlsWnd::CreateControlsWindow(hInstance(), _wm,
                                                      hWnd());
+            // This needs to access the media player HWND, therefore we need
+            // to wait for the vout to be created
+            VP()->get_mp().eventManager().onVout([this](int nbVout) {
+                if ( nbVout == 0 )
+                    return;
+                HWND hwnd = FindMP_hWnd();
+                if ( hwnd == nullptr )
+                    return;
+                DWORD s = GetWindowLong(hwnd, GWL_STYLE);
+                s |= WS_CLIPSIBLINGS;
+                SetWindowLong(hwnd, GWL_STYLE, s);
+                // Now we can hook onto the media player's HWND:
+                MouseHook(true);
+            });
             break;
         }
         case WM_PAINT:{
             PAINTSTRUCT PaintStruct;
             HDC hDC = BeginPaint(hWnd(), &PaintStruct);
-            RECT rect;
-            GetClientRect(hWnd(), &rect);
-            int IconX = ((rect.right - rect.left) - GetSystemMetrics(SM_CXICON))/2;
-            int IconY = ((rect.bottom - rect.top) - GetSystemMetrics(SM_CYICON))/2;
-            DrawIcon(hDC, IconX, IconY, RC().hBackgroundIcon);
+            if( PO() && PO()->get_enable_branding() )
+            {
+                RECT rect;
+                GetClientRect(hWnd(), &rect);
+                int IconX = ((rect.right - rect.left) - GetSystemMetrics(SM_CXICON))/2;
+                int IconY = ((rect.bottom - rect.top) - GetSystemMetrics(SM_CYICON))/2;
+                DrawIcon(hDC, IconX, IconY, RC().hBackgroundIcon);
+            }
             EndPaint(hWnd(), &PaintStruct);
             break;
         }
         case WM_SHOWWINDOW:{
             if(FALSE!=wParam){ //showing
-                NeedShowControls();
+                _CtrlsWnd->NeedShowControls();
             }
             break;
         }
-        case WM_TRY_SET_MOUSE_HOOK:{
-            MouseHook(true);
-            break;
-        }
         case WM_SIZE:
             if(_CtrlsWnd){
                 int new_client_width = LOWORD(lParam);
@@ -685,13 +603,26 @@ LRESULT VLCHolderWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
             }
             break;
         case WM_MOUSEMOVE:
+            if (_CtrlsWnd && (_oldMouseCoords.x != GET_X_LPARAM(lParam) ||
+                _oldMouseCoords.y != GET_Y_LPARAM(lParam)))
+            {
+                _CtrlsWnd->NeedShowControls();
+                _oldMouseCoords = MAKEPOINTS(lParam);
+            }
+            break;
         case WM_LBUTTONDBLCLK:
             WM().OnMouseEvent(uMsg);
             break;
         case WM_MOUSE_EVENT_NOTIFY:{
-            WM().OnMouseEvent(wParam);
+            // This is called synchronously, though handling it directly from the mouse hook
+            // deadlocks (quite likely because we're destroying the windows we're being called from)
+            // Just repost this asynchronously, and let the called know we received the message.
+            PostMessage(hWnd(), WM_MOUSE_EVENT_REPOST, wParam, lParam);
             return WM_MOUSE_EVENT_NOTIFY_SUCCESS;
         }
+        case WM_MOUSE_EVENT_REPOST:
+            WM().OnMouseEvent(wParam);
+            break;
         default:
             return VLCWnd::WindowProc(uMsg, wParam, lParam);
     }
@@ -700,8 +631,6 @@ LRESULT VLCHolderWnd::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
 
 void VLCHolderWnd::DestroyWindow()
 {
-    LibVlcDetach();
-
     if(_CtrlsWnd){
         delete _CtrlsWnd;
         _CtrlsWnd = 0;
@@ -709,7 +638,7 @@ void VLCHolderWnd::DestroyWindow()
 
     if(hWnd())
         ::DestroyWindow(hWnd());
-};
+}
 
 LRESULT CALLBACK VLCHolderWnd::MouseHookProc(int nCode, WPARAM wParam, LPARAM lParam)
 {
@@ -775,46 +704,6 @@ HWND VLCHolderWnd::FindMP_hWnd()
         return GetWindow(hWnd(), GW_CHILD);
 }
 
-//libvlc events arrives from separate thread
-void VLCHolderWnd::OnLibVlcEvent(const libvlc_event_t* event)
-{
-    //We need set hook to catch doubleclicking (to switch to fullscreen and vice versa).
-    //But libvlc media window may not exist yet,
-    //and we don't know when it will be created, nor ThreadId of it.
-    //So we try catch events,
-    //(suppose wnd will be ever created),
-    //and then try set mouse hook.
-    HWND hMPWnd = FindMP_hWnd();
-    const DWORD WndThreadID = (hMPWnd) ? GetWindowThreadProcessId(hMPWnd, NULL) : 0;
-    //if no hook, or window thread has changed
-    if(!_hMouseHook || (hMPWnd && WndThreadID != _MouseHookThreadId)){
-        DWORD s = GetWindowLong(hMPWnd, GWL_STYLE);
-        s |= WS_CLIPSIBLINGS;
-        SetWindowLong(hMPWnd, GWL_STYLE, s);
-
-        //libvlc events arrives from separate thread,
-        //so we need post message to main thread, to notify it.
-        PostMessage(hWnd(), WM_TRY_SET_MOUSE_HOOK, 0, 0);
-    }
-
-    if( _CtrlsWnd )
-        _CtrlsWnd->OnLibVlcEvent(event);
-}
-
-void VLCHolderWnd::LibVlcAttach()
-{
-    if( VP() )
-        libvlc_media_player_set_hwnd( VP()->get_mp(), hWnd() );
-}
-
-void VLCHolderWnd::LibVlcDetach()
-{
-    if( VP() )
-        libvlc_media_player_set_hwnd( VP()->get_mp(), 0);
-
-    MouseHook(false);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 //VLCFullScreenWnd members
 ////////////////////////////////////////////////////////////////////////////////
@@ -912,7 +801,7 @@ LRESULT CALLBACK VLCFullScreenWnd::FSWndWindowProc(HWND hWnd, UINT uMsg, WPARAM
 VLCFullScreenWnd* VLCFullScreenWnd::CreateFSWindow(VLCWindowsManager* WM)
 {
     HWND hWnd = CreateWindow(getClassName(),
-                TEXT("VLC ActiveX Full Screen Window"),
+                TEXT("VLC Full Screen Window"),
                 WS_POPUP|WS_CLIPCHILDREN,
                 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
                 0,
@@ -930,8 +819,8 @@ VLCFullScreenWnd* VLCFullScreenWnd::CreateFSWindow(VLCWindowsManager* WM)
 //VLCWindowsManager
 ///////////////////////
 VLCWindowsManager::VLCWindowsManager(HMODULE hModule, const VLCViewResources& rc,
-                                     const vlc_player_options* po)
-    :_rc(rc), _hModule(hModule), _po(po), _hWindowedParentWnd(0), _vp(0),
+                                     vlc_player* player, const vlc_player_options* po)
+    :_rc(rc), _hModule(hModule), _po(po), _hWindowedParentWnd(0), _vp(player),
     _HolderWnd(0), _FSWnd(0), _b_new_messages_flag(false), Last_WM_MOUSEMOVE_Pos(0)
 {
     VLCFullScreenWnd::RegisterWndClassName(hModule);
@@ -967,34 +856,6 @@ void VLCWindowsManager::DestroyWindows()
     _FSWnd = 0;
 }
 
-void VLCWindowsManager::LibVlcAttach(vlc_player* vp)
-{
-    if(!_HolderWnd)
-        return;//VLCWindowsManager::CreateWindows was not called
-
-    if( vp && _vp != vp ){
-        LibVlcDetach();
-    }
-
-    if( !_vp ){
-        _vp = vp;
-        VlcEvents(true);
-    }
-
-    _HolderWnd->LibVlcAttach();
-}
-
-void VLCWindowsManager::LibVlcDetach()
-{
-    if(_HolderWnd)
-        _HolderWnd->LibVlcDetach();
-
-    if(_vp){
-        VlcEvents(false);
-        _vp = 0;
-    }
-}
-
 void VLCWindowsManager::StartFullScreen()
 {
     if( !_HolderWnd || ( PO() && !PO()->get_enable_fs() ) )
@@ -1047,21 +908,21 @@ void VLCWindowsManager::EndFullScreen()
 
         ShowWindow(_FSWnd->getHWND(), SW_HIDE);
 
-        if(_FSWnd){
-            _FSWnd->DestroyWindow();
-        }
-        _FSWnd = 0;
+        _FSWnd->DestroyWindow();
+        _FSWnd = nullptr;
    }
 }
 
 void VLCWindowsManager::ToggleFullScreen()
 {
-    if(IsFullScreen()){
+    auto isFullscreen = IsFullScreen();
+    if ( isFullscreen ){
         EndFullScreen();
     }
     else{
         StartFullScreen();
     }
+    _HolderWnd->ControlWindow()->UpdateFullscreenButton( !isFullscreen );
 }
 
 bool VLCWindowsManager::IsFullScreen()
@@ -1075,6 +936,7 @@ void VLCWindowsManager::OnKeyDownEvent(UINT uKeyMsg)
         case VK_ESCAPE:
         case 'F':
             EndFullScreen();
+            _HolderWnd->ControlWindow()->UpdateFullscreenButton( FALSE );
             break;
     }
 }
@@ -1089,64 +951,10 @@ void VLCWindowsManager::OnMouseEvent(UINT uMouseMsg)
             DWORD MsgPos = GetMessagePos();
             if(Last_WM_MOUSEMOVE_Pos != MsgPos){
                 Last_WM_MOUSEMOVE_Pos = MsgPos;
-                _HolderWnd->NeedShowControls();
+                _HolderWnd->ControlWindow()->NeedShowControls();
             }
             break;
         }
     }
 }
 
-//libvlc events arrives from separate thread
-void VLCWindowsManager::OnLibVlcEvent_proxy(const libvlc_event_t* event, void *param)
-{
-    VLCWindowsManager* WM = static_cast<VLCWindowsManager*>(param);
-    WM->OnLibVlcEvent(event);
-}
-
-void VLCWindowsManager::VlcEvents(bool Attach)
-{
-    if( !VP() )
-        return;
-
-    vlc_player& vp = *VP();
-
-    libvlc_event_manager_t* em =
-        libvlc_media_player_event_manager( vp.get_mp() );
-    if(!em)
-        return;
-
-    for(int e=libvlc_MediaPlayerMediaChanged; e<=libvlc_MediaPlayerVout; ++e){
-        switch(e){
-        //case libvlc_MediaPlayerMediaChanged:
-        //case libvlc_MediaPlayerNothingSpecial:
-        //case libvlc_MediaPlayerOpening:
-        //case libvlc_MediaPlayerBuffering:
-        case libvlc_MediaPlayerPlaying:
-        case libvlc_MediaPlayerPaused:
-        case libvlc_MediaPlayerStopped:
-        //case libvlc_MediaPlayerForward:
-        //case libvlc_MediaPlayerBackward:
-        //case libvlc_MediaPlayerEndReached:
-        //case libvlc_MediaPlayerEncounteredError:
-        //case libvlc_MediaPlayerTimeChanged:
-        case libvlc_MediaPlayerPositionChanged:
-        //case libvlc_MediaPlayerSeekableChanged:
-        //case libvlc_MediaPlayerPausableChanged:
-        //case libvlc_MediaPlayerTitleChanged:
-        //case libvlc_MediaPlayerSnapshotTaken:
-        //case libvlc_MediaPlayerLengthChanged:
-        //case libvlc_MediaPlayerVout:
-            if(Attach)
-                libvlc_event_attach(em, e, OnLibVlcEvent_proxy, this);
-            else
-                libvlc_event_detach(em, e, OnLibVlcEvent_proxy, this);
-            break;
-        }
-    }
-}
-
-void VLCWindowsManager::OnLibVlcEvent(const libvlc_event_t* event)
-{
-    if(_HolderWnd) _HolderWnd->OnLibVlcEvent(event);
-    if(_FSWnd) _FSWnd->OnLibVlcEvent(event);
-}
