@@ -9,7 +9,7 @@ info()
 }
 
 ARCH="x86_64"
-MINIMAL_OSX_VERSION="10.6"
+MINIMAL_OSX_VERSION="10.7"
 OSX_VERSION="10.9"
 SDKROOT=`xcode-select -print-path`/Platforms/MacOSX.platform/Developer/SDKs/MacOSX$OSX_VERSION.sdk
 VERBOSE=no
@@ -85,9 +85,9 @@ builddir=`pwd`
 info "Building in \"$builddir\""
 
 export CC="xcrun clang -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION}"
-export CXX="xcrun clang++ -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION}"
+export CXX="xcrun clang++ -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION} -stdlib=libc++"
 export OBJC="xcrun clang -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION}"
-export OBJCXX="xcrun clang++ -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION}"
+export OBJCXX="xcrun clang++ -isysroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION} -stdlib=libc++"
 export LD="xcrun ld -syslibroot ${SDKROOT} -mmacosx-version-min=${MINIMAL_OSX_VERSION}"
 
 export PATH="${npapiroot}/extras/macosx/vlc/extras/tools/build/bin:${npapiroot}/extras/macosx/vlc/contrib/${ARCH}-apple-darwin10/bin:$PATH"
@@ -100,6 +100,8 @@ info "Bootstrapping"
 ./autogen.sh
 fi
 
+git submodule update --init
+
 info "Creating builddir and configuration"
 mkdir -p ${ARCH}-build && cd ${ARCH}-build
 ../configure --build=${ARCH}-apple-darwin10
