@@ -60,6 +60,10 @@
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
+#ifdef XP_WIN
+# include <windows.h>
+#endif
+
 // Include stdint before NP*.h
 #include <stdint.h>
 
@@ -77,8 +81,6 @@
     typedef int32_t NPint32_t;
 #endif
 
-#include "locking.h"
-
 /*
  * Define PLUGIN_TRACE to have the wrapper functions print
  * messages to stderr whenever they are called.
