@@ -11,6 +11,7 @@
  *          Yannick Brehon <y.brehon@qiplay.com>
  *          Felix Paul Kühne <fkuehne # videolan.org>
  *          Ludovic Fauvet <etix@videolan.org>
+ *          Hugo Beauzée-Luyssen <hugo@beauzee.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,13 +53,11 @@ VlcPluginBase::VlcPluginBase( NPP instance, NPuint16_t mode ) :
     i_npmode(mode),
     b_stream(0),
     psz_target(NULL),
-    libvlc_instance(NULL),
     p_scriptClass(NULL),
     p_browser(instance),
     psz_baseURL(NULL)
 {
     memset(&npwindow, 0, sizeof(NPWindow));
-    _instances.insert(this);
 }
 
 static bool boolValue(const char *value) {
@@ -68,30 +67,6 @@ static bool boolValue(const char *value) {
              !strcasecmp(value, "yes") );
 }
 
-std::set<VlcPluginBase*> VlcPluginBase::_instances;
-
-void VlcPluginBase::eventAsync(void *param)
-{
-    VlcPluginBase *plugin = (VlcPluginBase*)param;
-    if( _instances.find(plugin) == _instances.end() )
-        return;
-
-    plugin->events.deliver(plugin->getBrowser());
-    plugin->update_controls();
-}
-
-void VlcPluginBase::event_callback(const libvlc_event_t* event,
-                NPVariant *npparams, uint32_t npcount)
-{
-#if defined(XP_UNIX) || defined(XP_WIN) || defined (XP_MACOSX)
-    events.callback(event, npparams, npcount);
-    NPN_PluginThreadAsyncCall(getBrowser(), eventAsync, this);
-#else
-#   warning NPN_PluginThreadAsyncCall not implemented yet.
-    printf("No NPN_PluginThreadAsyncCall(), doing nothing.\n");
-#endif
-}
-
 NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
 {
     /* prepare VLC command line */
@@ -136,7 +111,11 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
 #endif
 
     /* common settings */
+#ifndef NDEBUG
     ppsz_argv[ppsz_argc++] = "-vv";
+#else
+    ppsz_argv[ppsz_argc++] = "--quiet";
+#endif
     ppsz_argv[ppsz_argc++] = "--no-stats";
     ppsz_argv[ppsz_argc++] = "--no-media-library";
     ppsz_argv[ppsz_argc++] = "--intf=dummy";
@@ -144,6 +123,8 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
     ppsz_argv[ppsz_argc++] = "--no-xlib";
 
     bool b_autoloop = false;
+    bool b_mute = false;
+    int i_volume = -1;
 
     /* parse plugin arguments */
     for( int i = 0; (i < argc) && (ppsz_argc < MAX_PARAMS); i++ )
@@ -164,7 +145,7 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
         else if( !strcmp( argn[i], "autoplay")
               || !strcmp( argn[i], "autostart") )
         {
-            set_autoplay(boolValue(argv[i]));
+            set_autoplay( boolValue(argv[i]) );
         }
         else if( !strcmp( argn[i], "fullscreen" )
               || !strcmp( argn[i], "allowfullscreen" )
@@ -174,10 +155,11 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
         }
         else if( !strcmp( argn[i], "mute" ) )
         {
-            if( boolValue(argv[i]) )
-            {
-                ppsz_argv[ppsz_argc++] = "--volume=0";
-            }
+            b_mute = boolValue( argv[i] );
+        }
+        else if( !strcmp( argn[i], "volume" ) )
+        {
+            i_volume = atoi( argv[i] );
         }
         else if( !strcmp( argn[i], "loop")
               || !strcmp( argn[i], "autoloop") )
@@ -199,14 +181,22 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
         }
     }
 
-    libvlc_instance = libvlc_new(ppsz_argc, ppsz_argv);
-    if( !libvlc_instance )
+    try {
+        VLC::Instance instance( ppsz_argc, ppsz_argv );
+        m_player.open(instance);
+    }
+    catch (std::runtime_error&) {
         return NPERR_GENERIC_ERROR;
+    }
 
-    vlc_player::open(libvlc_instance);
+    m_player.mlp().setPlaybackMode( b_autoloop ? libvlc_playback_mode_loop :
+                                      libvlc_playback_mode_default );
 
-    vlc_player::set_mode(b_autoloop ? libvlc_playback_mode_loop :
-                                      libvlc_playback_mode_default);
+    if( b_mute )
+        m_player.get_mp().setMute( true );
+
+    if( i_volume >= 0 && i_volume <= 200 )
+        m_player.get_mp().setVolume( i_volume );
 
     /*
     ** fetch plugin base URL, which is the URL of the page containing the plugin
@@ -256,12 +246,10 @@ NPError VlcPluginBase::init(int argc, char* const argn[], char* const argv[])
     /* new APIs */
     p_scriptClass = RuntimeNPClass<LibvlcRootNPObject>::getClass();
 
-    libvlc_media_player_t *p_md = getMD();
-    if( p_md ) {
-      libvlc_event_manager_t *p_em;
-      p_em = libvlc_media_player_event_manager( getMD() );
-      events.hook_manager( p_em, this );
-    }
+    // Update the UI if required when we switch media
+    m_player.mlp().eventManager().onNextItemSet([this](VLC::MediaPtr) {
+        update_controls();
+    });
 
     return NPERR_NO_ERROR;
 }
@@ -270,19 +258,6 @@ VlcPluginBase::~VlcPluginBase()
 {
     free(psz_baseURL);
     free(psz_target);
-
-    if( vlc_player::is_open() )
-    {
-
-        if( playlist_isplaying() )
-            playlist_stop();
-        events.unhook_manager( this );
-        vlc_player::close();
-    }
-    if( libvlc_instance )
-        libvlc_release( libvlc_instance );
-
-    _instances.erase(this);
 }
 
 void VlcPluginBase::setWindow(const NPWindow &window)
@@ -297,20 +272,192 @@ NPError VlcPluginBase::get_root_layer(void *value)
 }
 #endif
 
-bool VlcPluginBase::handle_event(void *event)
+bool VlcPluginBase::handle_event(void *)
 {
     return false;
 }
 
-/*****************************************************************************
- * VlcPluginBase playlist replacement methods
- *****************************************************************************/
-bool  VlcPluginBase::player_has_vout()
+template <typename... Args>
+struct AsyncEventWrapper
 {
-    bool r = false;
-    if( playlist_isplaying() )
-        r = libvlc_media_player_has_vout(get_mp())!=0;
-    return r;
+    template <typename... PFArgs>
+    AsyncEventWrapper(NPP b, NPObject* l, PFArgs&&... args)
+        : browser( b )
+        , listener( l )
+        , args( std::make_tuple( std::forward<Args>( args )... ) )
+    {
+    }
+
+    NPP browser;
+    NPObject* listener;
+    // If we do not decay Args here, we will end up declaring a tuple holding references
+    // The tuple returned by make_tuple does that, but we end up converting from the correct tuple
+    // type to an invalid tuple type, since we would end up using dangling references.
+    std::tuple<typename std::decay<Args>::type...> args;
+};
+
+template <typename... Args>
+static void invokeEvent( NPP browser, NPObject* listener, Args&&... args )
+{
+    auto wrapper = new AsyncEventWrapper<Args...>( browser, listener, std::forward<Args>( args )... );
+    NPN_PluginThreadAsyncCall( browser, [](void* data) {
+        auto w = reinterpret_cast<AsyncEventWrapper<Args...>*>( data );
+        auto args = npapi::wrap( w->args );
+        NPVariant result;
+        if (NPN_InvokeDefault( w->browser, w->listener, reinterpret_cast<NPVariant*>( args.data() ), args.size(), &result ))
+        {
+            // "result" content is unspecified when invoke fails. Don't clean potential garbage
+            NPN_ReleaseVariantValue( &result );
+        }
+        delete w;
+    }, wrapper);
+}
+
+class CallbackClosure
+{
+public:
+    CallbackClosure(NPP browser, npapi::Variant&& listener)
+        : _browser( browser )
+        , _listener( std::move( listener ) )
+    {
+    }
+
+    CallbackClosure(const CallbackClosure&) = delete;
+
+#ifndef _MSC_VER
+    CallbackClosure(CallbackClosure&&) = default;
+#else
+    CallbackClosure(CallbackClosure&& c)
+        : _browser( std::move(c._browser) )
+        , _listener( std::move(c._listener) )
+    {
+    }
+#endif
+
+    template <typename... Args>
+    void operator()(Args&&... params) const
+    {
+        // This is expected to receive a copy of the listener
+        invokeEvent( _browser, _listener, std::forward<Args>( params )... );
+    }
+
+    void operator()(VLC::MediaPtr) const
+    {
+        // Force Media to be ignored since we don't have a binding for it.
+        // This is expected to receive a copy of the listener
+        invokeEvent( _browser, _listener );
+    }
+private:
+    NPP _browser;
+    npapi::Variant _listener;
+};
+
+static struct vlcevents_t {
+    const char* name;
+    libvlc_event_type_t type;
+} vlcevents[] = {
+    { "MediaPlayerMediaChanged", libvlc_MediaPlayerMediaChanged },
+    { "MediaPlayerNothingSpecial", libvlc_MediaPlayerNothingSpecial },
+    { "MediaPlayerOpening", libvlc_MediaPlayerOpening },
+    { "MediaPlayerBuffering", libvlc_MediaPlayerBuffering },
+    { "MediaPlayerPlaying", libvlc_MediaPlayerPlaying },
+    { "MediaPlayerPaused", libvlc_MediaPlayerPaused },
+    { "MediaPlayerStopped", libvlc_MediaPlayerStopped },
+    { "MediaPlayerForward", libvlc_MediaPlayerForward },
+    { "MediaPlayerBackward", libvlc_MediaPlayerBackward },
+    { "MediaPlayerEndReached", libvlc_MediaPlayerEndReached },
+    { "MediaPlayerEncounteredError", libvlc_MediaPlayerEncounteredError },
+    { "MediaPlayerTimeChanged", libvlc_MediaPlayerTimeChanged },
+    { "MediaPlayerPositionChanged", libvlc_MediaPlayerPositionChanged },
+    { "MediaPlayerSeekableChanged", libvlc_MediaPlayerSeekableChanged },
+    { "MediaPlayerPausableChanged", libvlc_MediaPlayerPausableChanged },
+    { "MediaPlayerTitleChanged", libvlc_MediaPlayerTitleChanged },
+    { "MediaPlayerLengthChanged", libvlc_MediaPlayerLengthChanged },
+};
+
+void VlcPluginBase::subscribe(const char* eventName, npapi::Variant listener)
+{
+    auto event = std::find_if(std::begin(vlcevents), std::end(vlcevents), [eventName](const vlcevents_t& e) {
+        return !strcmp( e.name, eventName);
+    });
+    if (event == std::end(vlcevents))
+        return;
+
+    auto listenerRaw = (NPObject*)listener;
+    auto closure = CallbackClosure{ p_browser, std::move( listener ) };
+
+    VLC::EventManager::RegisteredEvent e = nullptr;
+    switch ( event->type )
+    {
+        case libvlc_MediaPlayerNothingSpecial:
+            e = player().get_mp().eventManager().onNothingSpecial( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerOpening:
+            e = player().get_mp().eventManager().onOpening( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerPlaying:
+            e = player().get_mp().eventManager().onPlaying( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerPaused:
+            e = player().get_mp().eventManager().onPaused( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerStopped:
+            e = player().get_mp().eventManager().onStopped( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerForward:
+            e = player().get_mp().eventManager().onForward( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerBackward:
+            e = player().get_mp().eventManager().onBackward( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerEndReached:
+            e = player().get_mp().eventManager().onEndReached( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerEncounteredError:
+            e = player().get_mp().eventManager().onEncounteredError( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerBuffering:
+            e = player().get_mp().eventManager().onBuffering( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerTimeChanged:
+            e = player().get_mp().eventManager().onTimeChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerMediaChanged:
+            e = player().get_mp().eventManager().onMediaChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerPositionChanged:
+            e = player().get_mp().eventManager().onPositionChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerSeekableChanged:
+            e = player().get_mp().eventManager().onSeekableChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerPausableChanged:
+            e = player().get_mp().eventManager().onPausableChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerTitleChanged:
+            e = player().get_mp().eventManager().onTitleChanged( std::move( closure ) );
+            break;
+        case libvlc_MediaPlayerLengthChanged:
+            e = player().get_mp().eventManager().onLengthChanged( std::move( closure ) );
+            break;
+        default:
+            break;
+    }
+    if ( e != nullptr )
+    {
+        m_events.emplace_back( std::string(eventName), listenerRaw, e );
+    }
+}
+
+void VlcPluginBase::unsubscribe(const char* eventName, npapi::Variant listener)
+{
+    auto event = std::find_if( begin( m_events ), end( m_events ), [eventName, listener](const decltype(m_events)::value_type e) {
+        return std::get<0>( e ) == eventName && std::get<1>( e ) == listener;
+    });
+    if ( event == end( m_events ) )
+        return;
+    std::get<2>( *event )->unregister();
+    m_events.erase( event );
 }
 
 /*****************************************************************************
@@ -477,19 +624,19 @@ void VlcPluginBase::control_handler(vlc_toolbar_clicked_t clicked)
     {
         case clicked_Play:
         {
-            playlist_play();
+            player().play();
         }
         break;
 
         case clicked_Pause:
         {
-            playlist_pause();
+            player().get_mp().pause();
         }
         break;
 
         case clicked_Stop:
         {
-            playlist_stop();
+            player().get_mp().stop();
         }
         break;
 
