@@ -32,10 +32,11 @@
 #define __VLCPLUGIN_BASE_H__
 
 #include "common.h"
-#include "events.h"
+#include "utils.hpp"
 
 #include <vector>
-#include <set>
+#include <utility>
+#include <unordered_map>
 
 #include "../common/vlc_player_options.h"
 #include "../common/vlc_player.h"
@@ -54,7 +55,7 @@ typedef enum vlc_toolbar_clicked_e {
     clicked_Unmute
 } vlc_toolbar_clicked_t;
 
-class VlcPluginBase: private vlc_player_options, private vlc_player
+class VlcPluginBase: private vlc_player_options
 {
 protected:
 
@@ -62,11 +63,6 @@ public:
     VlcPluginBase( NPP, NPuint16_t );
     virtual ~VlcPluginBase();
 
-    vlc_player& get_player()
-    {
-        return *static_cast<vlc_player*>(this);
-    }
-
     vlc_player_options& get_options()
         { return *static_cast<vlc_player_options*>(this); }
     const vlc_player_options& get_options() const
@@ -74,13 +70,9 @@ public:
 
     NPError             init(int argc, char* const argn[], char* const argv[]);
 
-    libvlc_media_player_t* getMD()
+    VLC::MediaPlayer& getMD()
     {
-        if( !is_open() )
-        {
-             libvlc_printerr("no mediaplayer");
-        }
-        return get_mp();
+        return m_player.get_mp();
     }
 
     NPP                 getBrowser() { return p_browser; };
@@ -97,68 +89,12 @@ public:
     int      b_stream;
     char *   psz_target;
 
-    void playlist_play()
-    {
-        play();
-    }
-    void playlist_play_item(int idx)
-    {
-        play(idx);
-    }
-    void playlist_stop()
-    {
-        stop();
-    }
-    void playlist_next()
-    {
-        next();
-    }
-    void playlist_prev()
-    {
-        prev();
-    }
-    void playlist_pause()
-    {
-        pause();
-    }
-    void playlist_togglePause()
-    {
-        togglePause();
-    }
-    int playlist_isplaying()
-    {
-        return is_playing();
-    }
-    int playlist_currentitem()
-    {
-        return current_item();
-    }
-    int playlist_add( const char * mrl)
-    {
-        return add_item(mrl);
-    }
-    int playlist_add_extended_untrusted( const char *mrl, const char *,
-                    int optc, const char **optv )
-    {
-        return add_item(mrl, optc, optv);
-    }
-    int playlist_delete_item( int idx)
-    {
-        return delete_item(idx);
-    }
-    void playlist_clear()
-    {
-        clear_items() ;
-    }
-    int  playlist_count()
-    {
-        return items_count();
-    }
-    bool playlist_select(int);
-
     void control_handler(vlc_toolbar_clicked_t);
 
-    bool  player_has_vout();
+    vlc_player& player()
+    {
+        return m_player;
+    }
 
     virtual bool create_windows() = 0;
     virtual bool resize_windows() = 0;
@@ -172,7 +108,7 @@ public:
 
     virtual void toggle_fullscreen() = 0;
     virtual void set_fullscreen(int) = 0;
-    virtual int get_fullscreen() = 0;
+    virtual bool get_fullscreen() = 0;
 
     virtual void set_toolbar_visible(bool) = 0;
     virtual bool get_toolbar_visible() = 0;
@@ -184,17 +120,9 @@ public:
 
     static bool canUseEventListener();
 
-    EventObj events;
-    void event_callback(const libvlc_event_t *, NPVariant *, uint32_t);
-
+    void subscribe(const char* eventName, npapi::Variant listener);
+    void unsubscribe( const char* eventName, npapi::Variant listener );
 protected:
-    // called after libvlc_media_player_new_from_media
-    virtual void on_media_player_new()     {};
-    // called before libvlc_media_player_release
-    virtual void on_media_player_release() {};
-
-    /* VLC reference */
-    libvlc_instance_t   *libvlc_instance;
     NPClass             *p_scriptClass;
 
     /* browser reference */
@@ -204,10 +132,10 @@ protected:
     /* display settings */
     NPWindow  npwindow;
 
-    static void eventAsync(void *);
+    vlc_player m_player;
 
 private:
-    static std::set<VlcPluginBase*> _instances;
+    std::vector<std::tuple<std::string, NPObject*, VLC::EventManager::RegisteredEvent>> m_events;
 };
 
 #endif
