@@ -37,14 +37,13 @@ VlcPluginGtk::VlcPluginGtk(NPP instance, NPuint16_t mode) :
     parent(NULL),
     parent_vbox(NULL),
     video_container(NULL),
+    fullscreen_win(NULL),
     toolbar(NULL),
     time_slider(NULL),
     vol_slider(NULL),
-    fullscreen_win(NULL),
     is_fullscreen(false),
     is_toolbar_visible(false),
-    time_slider_timeout_id(0),
-    vol_slider_timeout_id(0)
+    m_timer_update_timeout( 0 )
 {
     memset(&video_xwindow, 0, sizeof(Window));
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
@@ -57,13 +56,17 @@ VlcPluginGtk::VlcPluginGtk(NPP instance, NPuint16_t mode) :
 
 VlcPluginGtk::~VlcPluginGtk()
 {
+    {
+        std::lock_guard<std::mutex> lock( m_timer_lock );
+        if ( m_timer_update_timeout != 0 )
+            g_source_remove( m_timer_update_timeout );
+    }
 }
 
 void VlcPluginGtk::set_player_window()
 {
-    libvlc_media_player_set_xwindow(get_player().get_mp(),
-                                    video_xwindow);
-    libvlc_video_set_mouse_input(get_player().get_mp(), 0);
+    player().get_mp().setXwindow( video_xwindow);
+    player().get_mp().setMouseInput( false );
 }
 
 void VlcPluginGtk::toggle_fullscreen()
@@ -118,7 +121,7 @@ void VlcPluginGtk::set_fullscreen(int yes)
     }
 }
 
-int VlcPluginGtk::get_fullscreen()
+bool VlcPluginGtk::get_fullscreen()
 {
     return is_fullscreen;
 }
@@ -173,9 +176,9 @@ static void toolbar_handler(GtkToolButton *btn, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     const gchar *stock_id = gtk_tool_button_get_stock_id(btn);
-    for (int i = 0; i < sizeof(tool_actions)/sizeof(tool_actions_t); ++i) {
-        if (!strcmp(stock_id, tool_actions[i].stock_id)) {
-            plugin->control_handler(tool_actions[i].clicked);
+    for (auto a : tool_actions ) {
+        if (!strcmp(stock_id, a.stock_id)) {
+            plugin->control_handler(a.clicked);
             return;
         }
     }
@@ -190,9 +193,10 @@ static void menu_handler(GtkMenuItem *menuitem, gpointer user_data)
         plugin->set_toolbar_visible(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
         return;
     }
-    for (int i = 0; i < sizeof(tool_actions)/sizeof(tool_actions_t); ++i) {
-        if (!strcmp(stock_id, tool_actions[i].stock_id)) {
-            plugin->control_handler(tool_actions[i].clicked);
+    for (auto a : tool_actions)
+    {
+        if (!strcmp(stock_id, a.stock_id)) {
+            plugin->control_handler(a.clicked);
             return;
         }
     }
@@ -207,7 +211,7 @@ void VlcPluginGtk::popup_menu()
 
     /* play/pause */
     menuitem = gtk_image_menu_item_new_from_stock(
-                        playlist_isplaying() ?
+                        player().mlp().isPlaying() ?
                         GTK_STOCK_MEDIA_PAUSE :
                         GTK_STOCK_MEDIA_PLAY, NULL);
     g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(menu_handler), this);
@@ -238,7 +242,7 @@ void VlcPluginGtk::popup_menu()
                    0, gtk_get_current_event_time());
 }
 
-static bool video_button_handler(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+static bool video_button_handler(GtkWidget*, GdkEventButton *event, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
@@ -251,21 +255,21 @@ static bool video_button_handler(GtkWidget *widget, GdkEventButton *event, gpoin
     return false;
 }
 
-static bool video_popup_handler(GtkWidget *widget, gpointer user_data)
+static bool video_popup_handler(GtkWidget*, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     plugin->popup_menu();
     return true;
 }
 
-static bool video_size_handler(GtkWidget *widget, GdkRectangle *rect, gpointer user_data)
+static bool video_size_handler(GtkWidget*, GdkRectangle *rect, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     plugin->resize_video_xwindow(rect);
     return true;
 }
 
-static bool video_expose_handler(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+static bool video_expose_handler(GtkWidget*, GdkEvent *event, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     GdkEventExpose *event_expose = (GdkEventExpose *) event;
@@ -303,53 +307,29 @@ static bool video_expose_handler(GtkWidget *widget, GdkEvent *event, gpointer us
     return true;
 }
 
-static gboolean do_time_slider_handler(gpointer user_data)
+gboolean VlcPluginGtk::update_time_slider(gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
-    libvlc_media_player_t *md = plugin->getMD();
-    if (md) {
-        gdouble value = gtk_range_get_value(GTK_RANGE(plugin->time_slider));
-        libvlc_media_player_set_position(md, value/100.0);
-    }
 
-    plugin->time_slider_timeout_id = 0;
+    std::lock_guard<std::mutex> lock( plugin->m_timer_lock );
+
+    auto pos = plugin->player().get_mp().position() * 100.0f;
+    gtk_range_set_value( GTK_RANGE( plugin->time_slider ), pos );
+    plugin->m_timer_update_timeout = 0;
     return FALSE;
 }
 
-static bool time_slider_handler(GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data)
+static bool time_slider_handler(GtkRange*, GtkScrollType, gdouble value, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
-    if (plugin->time_slider_timeout_id != 0)
-        return false;
-
-    plugin->time_slider_timeout_id = g_timeout_add(500,
-                                                  do_time_slider_handler,
-                                                  user_data);
+    plugin->getMD().setPosition( value / 100.0 );
     return false;
 }
 
-static gboolean do_vol_slider_handler(gpointer user_data)
+static bool vol_slider_handler(GtkRange*, GtkScrollType, gdouble value, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
-    libvlc_media_player_t *md = plugin->getMD();
-    if (md) {
-        gdouble value = gtk_range_get_value(GTK_RANGE(plugin->vol_slider));
-        libvlc_audio_set_volume(md, value);
-    }
-
-    plugin->vol_slider_timeout_id = 0;
-    return FALSE;
-}
-
-static bool vol_slider_handler(GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data)
-{
-    VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
-    if (plugin->vol_slider_timeout_id != 0)
-        return false;
-
-    plugin->vol_slider_timeout_id = g_timeout_add(100,
-                                                  do_vol_slider_handler,
-                                                  user_data);
+    plugin->getMD().setVolume( value );
     return false;
 }
 
@@ -359,13 +339,13 @@ static void fullscreen_win_visibility_handler(GtkWidget *widget, gpointer user_d
     plugin->do_set_fullscreen(gtk_widget_get_visible(widget));
 }
 
-static gboolean fullscreen_win_keypress_handler(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+static gboolean fullscreen_win_keypress_handler(GtkWidget*, GdkEventKey *event, gpointer user_data)
 {
     VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
     switch (event->keyval)
     {
     case GDK_KEY_space:
-        plugin->playlist_togglePause();
+        plugin->player().mlp().pause();
         return True;
     case GDK_KEY_Escape:
         plugin->set_fullscreen(false);
@@ -377,23 +357,18 @@ static gboolean fullscreen_win_keypress_handler(GtkWidget *widget, GdkEventKey *
 
 void VlcPluginGtk::update_controls()
 {
-    if (get_player().is_open()) {
-        libvlc_state_t state = libvlc_media_player_get_state(get_player().get_mp());
-        bool is_stopped = (state == libvlc_Stopped) ||
-                          (state == libvlc_Ended) ||
-                          (state == libvlc_Error);
-        if (is_stopped) {
-            XUnmapWindow(display, video_xwindow);
-        } else {
-            XMapWindow(display, video_xwindow);
-        }
+    auto state = player().get_mp().state();
+    if ( state == libvlc_Stopped || state == libvlc_Ended || state == libvlc_Error ) {
+        XUnmapWindow(display, video_xwindow);
+    } else {
+        XMapWindow(display, video_xwindow);
     }
 
     if (get_toolbar_visible()) {
         GtkToolItem *toolbutton;
 
         /* play/pause button */
-        const gchar *stock_id = playlist_isplaying() ? GTK_STOCK_MEDIA_PAUSE : GTK_STOCK_MEDIA_PLAY;
+        const gchar *stock_id = player().mlp().isPlaying() ? GTK_STOCK_MEDIA_PAUSE : GTK_STOCK_MEDIA_PLAY;
         toolbutton = gtk_toolbar_get_nth_item(GTK_TOOLBAR(toolbar), 0);
         if (strcmp(gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(toolbutton)), stock_id)) {
             gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(toolbutton), stock_id);
@@ -405,20 +380,14 @@ void VlcPluginGtk::update_controls()
         }
 
         /* toolbar sensitivity */
-        gtk_widget_set_sensitive(toolbar, get_player().is_open() );
+        gtk_widget_set_sensitive( toolbar, true );
 
         /* time slider */
-        if (!get_player().is_open() ||
-                !libvlc_media_player_is_seekable(get_player().get_mp())) {
+        if (!player().get_mp().isSeekable()) {
             gtk_widget_set_sensitive(time_slider, false);
             gtk_range_set_value(GTK_RANGE(time_slider), 0);
         } else {
             gtk_widget_set_sensitive(time_slider, true);
-            gdouble timepos = 100*libvlc_media_player_get_position(get_player().get_mp());
-            if (time_slider_timeout_id == 0) {
-                /* only set the time if the user is not dragging the slider */
-                gtk_range_set_value(GTK_RANGE(time_slider), timepos);
-            }
         }
 
         gtk_widget_show_all(toolbar);
@@ -495,6 +464,15 @@ bool VlcPluginGtk::create_windows()
     toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_MEDIA_PLAY);
     g_signal_connect(G_OBJECT(toolitem), "clicked", G_CALLBACK(toolbar_handler), this);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, -1);
+    m_player.get_mp().eventManager().onPaused([this, toolitem]() {
+        gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(toolitem), GTK_STOCK_MEDIA_PLAY);
+    });
+    m_player.get_mp().eventManager().onPlaying([this, toolitem]() {
+        gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(toolitem), GTK_STOCK_MEDIA_PAUSE);
+    });
+    m_player.get_mp().eventManager().onStopped([this, toolitem]() {
+        gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(toolitem), GTK_STOCK_MEDIA_PLAY);
+    });
     /* stop */
     toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_MEDIA_STOP);
     g_signal_connect(G_OBJECT(toolitem), "clicked", G_CALLBACK(toolbar_handler), this);
@@ -509,6 +487,11 @@ bool VlcPluginGtk::create_windows()
     gtk_container_add(GTK_CONTAINER(toolitem), time_slider);
     gtk_tool_item_set_expand(toolitem, true);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, -1);
+    m_player.get_mp().eventManager().onPositionChanged([this](float) {
+        std::lock_guard<std::mutex> lock( m_timer_lock );
+        if ( m_timer_update_timeout == 0 )
+            m_timer_update_timeout = g_timeout_add( 100, update_time_slider, (gpointer)this);
+    });
 
     /* volume slider */
     toolitem = gtk_tool_item_new();
