@@ -27,6 +27,7 @@
 #include "vlcplugin_base.h"
 
 #include <gtk/gtk.h>
+#include <mutex>
 #include <X11/Xlib.h>
 
 #define VLCPLUGINGTK_MENU_TOOLBAR "Show toolbar"
@@ -43,7 +44,7 @@ public:
 
     void toggle_fullscreen();
     void set_fullscreen(int);
-    int get_fullscreen();
+    bool get_fullscreen();
     void do_set_fullscreen(bool);
 
     void set_toolbar_visible(bool);
@@ -54,22 +55,25 @@ public:
     void resize_video_xwindow(GdkRectangle *rect);
 
     GdkPixbuf *cone_icon;
-    GtkWidget *time_slider, *vol_slider;
-    guint time_slider_timeout_id, vol_slider_timeout_id;
+
 private:
     void set_player_window();
     Display *get_display() { return display; }
+    static gboolean update_time_slider(gpointer user_data);
 
     unsigned int     i_width, i_height;
     GtkWidget *parent, *parent_vbox, *video_container;
     GtkWidget *fullscreen_win;
     GtkWidget *toolbar;
+    GtkWidget *time_slider, *vol_slider;
     gulong video_container_size_handler_id;
 
     Display *display;
     Window video_xwindow;
     XColor bg_color;
     bool is_fullscreen, is_toolbar_visible;
+    std::mutex m_timer_lock;
+    guint m_timer_update_timeout;
 };
 
 #endif /* __VLCPLUGIN_GTK_H__ */
