@@ -41,36 +41,30 @@ VlcPluginXcb::VlcPluginXcb(NPP instance, NPuint16_t mode) :
     memset(&video, 0, sizeof(Window));
 }
 
-VlcPluginXcb::~VlcPluginXcb()
-{
-}
-
 void VlcPluginXcb::set_player_window()
 {
-    libvlc_media_player_set_xwindow(get_player().get_mp(),
-                                    (uint32_t) video);
+    player().get_mp().setXwindow( (uint32_t) video );
 }
 
 void VlcPluginXcb::toggle_fullscreen()
 {
     if (!get_options().get_enable_fs()) return;
-    if (playlist_isplaying())
-        libvlc_toggle_fullscreen(get_player().get_mp());
+    if (player().mlp().isPlaying())
+        player().get_mp().toggleFullscreen();
 }
 
 void VlcPluginXcb::set_fullscreen(int yes)
 {
     if (!get_options().get_enable_fs()) return;
-    if (playlist_isplaying())
-        libvlc_set_fullscreen(get_player().get_mp(),yes);
+    if (player().mlp().isPlaying())
+        player().get_mp().setFullscreen( yes );
 }
 
-int  VlcPluginXcb::get_fullscreen()
+bool VlcPluginXcb::get_fullscreen()
 {
-    int r = 0;
-    if (playlist_isplaying())
-        r = libvlc_get_fullscreen(get_player().get_mp());
-    return r;
+    if (player().mlp().isPlaying())
+        return player().get_mp().fullscreen();
+    return 0;
 }
 
 bool VlcPluginXcb::create_windows()
@@ -148,4 +142,5 @@ bool VlcPluginXcb::destroy_windows()
     xcb_destroy_window(conn, parent);
     xcb_free_colors(conn, colormap, 0, 1, &colorpixel);
     xcb_disconnect(conn);
+    return true;
 }
