@@ -312,9 +312,6 @@ NPError NPP_Destroy( NPP instance, NPSavedData** )
 
     instance->pdata = NULL;
 
-    if( p_plugin->playlist_isplaying() )
-        p_plugin->playlist_stop();
-
     p_plugin->destroy_windows();
 
     delete p_plugin;
@@ -365,11 +362,11 @@ NPError NPP_SetWindow( NPP instance, NPWindow* window )
             {
                 if( p_plugin->psz_target )
                 {
-                    if( p_plugin->playlist_add( p_plugin->psz_target ) != -1 )
+                    if( p_plugin->player().add_item( p_plugin->psz_target ) != -1 )
                     {
                         if( p_plugin->get_options().get_autoplay() )
                         {
-                            p_plugin->playlist_play();
+                            p_plugin->player().play();
                         }
                     }
                     p_plugin->b_stream = true;
@@ -466,16 +463,16 @@ void NPP_StreamAsFile( NPP instance, NPStream *stream, const char* )
         return;
     }
 
-    if( p_plugin->playlist_add( stream->url ) != -1 )
+    if( p_plugin->player().add_item( stream->url ) != -1 )
     {
         if( p_plugin->get_options().get_autoplay() )
         {
-            p_plugin->playlist_play();
+            p_plugin->player().play();
         }
     }
 }
 
-void NPP_URLNotify( NPP instance, const char* ,
+void NPP_URLNotify( NPP, const char* ,
                     NPReason, void* )
 {
     /***** Insert NPP_URLNotify code here *****\
