@@ -249,7 +249,7 @@ bool VlcWindowlessMac::handle_event(void *event)
 
         CGContextClearRect(cgContext, CGRectMake(0, 0, npwindow.width, npwindow.height) );
 
-        if (m_media_width == 0 || m_media_height == 0 || (!lastFrame && !VlcPluginBase::playlist_isplaying()) || !get_player().is_open()) {
+        if (m_media_width == 0 || m_media_height == 0 || (!lastFrame && !player().mlp().isPlaying())) {
             drawNoPlayback(cgContext);
             return true;
         }
@@ -334,14 +334,3 @@ void VlcWindowlessMac::video_display_cb(void * /*picture*/)
             invalidate_window();
     }
 }
-
-void VlcWindowlessMac::set_player_window() {
-    libvlc_video_set_format_callbacks(getMD(),
-                                      video_format_proxy,
-                                      video_cleanup_proxy);
-    libvlc_video_set_callbacks(getMD(),
-                               video_lock_proxy,
-                               video_unlock_proxy,
-                               video_display_proxy,
-                               this);
-}
